/* w2ui 1.5.rc1 (nightly) (c) http://w2ui.com, vitmalina@gmail.com */
@font-face {
  font-family: "w2ui-font";
  src: url("data:application/x-font-woff;charset=utf-8;base64, ") format("woff");
  font-weight: normal;
  font-style: normal;
}
[class^="w2ui-icon-"]:before,
[class*=" w2ui-icon-"]:before {
  font-family: "w2ui-font";
  display: inline-block;
  vertical-align: middle;
  line-height: 1;
  font-weight: normal;
  font-style: normal;
  speak: none;
  text-decoration: inherit;
  text-transform: none;
  text-rendering: optimizeLegibility;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}
/* Icons */
.w2ui-icon-check:before {
  content: "\f101";
}
.w2ui-icon-columns:before {
  content: "\f102";
}
.w2ui-icon-cross:before {
  content: "\f103";
}
.w2ui-icon-empty:before {
  content: "\f104";
}
.w2ui-icon-info:before {
  content: "\f105";
}
.w2ui-icon-pencil:before {
  content: "\f106";
}
.w2ui-icon-plus:before {
  content: "\f107";
}
.w2ui-icon-reload:before {
  content: "\f108";
}
.w2ui-icon-search:before {
  content: "\f109";
}
/*************************************************
* --- Reset (used for all w2ui widgets)
* --- The reset is needed to coexist with other CSS
* --- on the same page (for example bootstrap)
*/
.w2ui-reset {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
  font-family: Verdana, Arial, sans-serif;
  font-size: 11px;
}
.w2ui-reset * {
  color: default;
  line-height: 100%;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
  margin: 0px;
  padding: 0px;
}
.w2ui-reset table {
  max-width: none;
  background-color: transparent;
  border-collapse: separate;
  border-spacing: 0;
  border: none;
}
.w2ui-reset table tr th,
.w2ui-reset table tr td {
  font-family: Verdana, Arial, sans-serif;
  font-size: 11px;
}
.w2ui-reset input:not([type=button]):not([type=submit]):not([type=checkbox]):not([type=radio]),
.w2ui-reset select,
.w2ui-reset textarea {
  display: inline-block;
  width: auto;
  height: auto;
  vertical-align: baseline;
  padding: 4px;
  margin: 0;
  font-size: 11px;
}
.w2ui-reset select {
  padding: 1px;
  height: 23px;
  font-size: 11px;
}
.w2ui-centered {
  position: absolute;
  left: 0px;
  right: 0px;
  top: 50%;
  -webkit-transform: translateY(-50%);
  -moz-transform: translateY(-50%);
  -ms-transform: translateY(-50%);
  -o-transform: translateY(-50%);
  transform: translateY(-50%);
  max-height: 100%;
  margin: 0px;
  padding: 0px 10px;
  text-align: center;
}
.w2ui-disabled,
.w2ui-readonly {
  background-color: #f1f1f1 !important;
  color: #777 !important;
}
.w2ui-message {
  font-size: 12px;
  position: absolute;
  z-index: 250;
  background-color: #f9f9f9;
  border: 1px solid #999999;
  box-shadow: 0px 0px 15px #aaaaaa;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
  border-top: 0px;
  border-radius: 0px 0px 6px 6px;
  overflow: auto;
}
.w2ui-message .w2ui-message-body {
  position: absolute;
  top: 0px;
  bottom: 45px;
  left: 0px;
  right: 0px;
  overflow: auto;
  line-height: 1.5;
  font-size: 14px;
}
.w2ui-message .w2ui-message-body .w2ui-centered {
  line-height: 1.5;
}
.w2ui-message .w2ui-message-buttons {
  position: absolute;
  height: 45px;
  bottom: 0px;
  left: 0px;
  right: 0px;
  border-top: 1px solid #e5e5e5;
  text-align: center;
  padding: 8px;
}
/*************************************************
* ---- Input Controls ----
*/
input:not([type=button]):not([type=submit]).w2ui-input,
textarea.w2ui-input {
  padding: 4px;
  border: 1px solid #cacaca;
  border-radius: 3px;
  color: #000000;
  background-color: #ffffff;
  line-height: normal;
}
input:not([type=button]):not([type=submit]).w2ui-input:focus,
textarea.w2ui-input:focus {
  outline-color: #72b2ff;
}
input:not([type=button]):not([type=submit]).w2ui-input:disabled,
textarea.w2ui-input:disabled,
input:not([type=button]):not([type=submit]).w2ui-input[readonly],
textarea.w2ui-input[readonly] {
  background-color: #f1f1f1;
  color: #777;
}
/* IE9-11 specific classes */
/* needs doblue :: */
input.w2ui-input::-ms-clear {
  display: none;
}
input.w2ui-input:-ms-input-placeholder {
  color: #aaa !important;
}
select.w2ui-input {
  color: #000000;
  padding: 0px 15px 0px 7px;
  line-height: 1.8;
  border-radius: 3px;
  border: 1px solid #cacaca;
  -webkit-appearance: none;
  background-image: url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAALCAQAAACnzwd+AAAAcklEQVR4AcXMsQFBQQDG4P9tAgC0gJYRQJZgKQMwCqCku6vVAAAA+NJHP4KHOk0aV2pRw61n4BBmyOxKQ8I4ehZeuhd3HTx6DQEGZ7sBfr2OOOOj3Yi43kMKs9sZknofOexqZ8npMygwWZTX51CipP+YA1OiZJbYYg9lAAAAAElFTkSuQmCC'), -webkit-linear-gradient(top, #ffffff 20%, #f6f6f6 50%, #eeeeee 52%, #f4f4f4 100%);
  background-size: 17px 6px, 100% 100%;
  background-position: right center, left top;
  background-repeat: no-repeat, no-repeat;
}
.w2ui-icon-expand:before {
  position: relative;
  top: 1px;
  left: 1px;
  content: ' ';
  width: 6px;
  height: 6px;
  border: 2px solid rgba(150, 150, 150, 0.8);
  border-bottom: 0;
  border-left: 0;
  transform: rotateZ(45deg);
}
.w2ui-icon-collapse:before {
  position: relative;
  top: -1px;
  left: 3px;
  content: ' ';
  width: 6px;
  height: 6px;
  border: 2px solid rgba(150, 150, 150, 0.8);
  border-bottom: 0;
  border-left: 0;
  transform: rotateZ(135deg);
}
/* On/Off switch */
input[type="checkbox"].w2ui-toggle {
  position: absolute;
  opacity: 0;
  width: 46px;
  height: 22px;
  padding: 0px;
  margin: 0px;
  margin-left: 2px;
}
/* Track */
input[type="checkbox"].w2ui-toggle + div {
  display: inline-block;
  width: 46px;
  height: 22px;
  border: 1px solid #bbb;
  border-radius: 30px;
  background-color: #eee;
  -webkit-transition-duration: .3s;
  -webkit-transition-property: background-color, box-shadow;
  -moz-transition-duration: .3s;
  -moz-transition-property: background-color, box-shadow;
  box-shadow: inset 0 0 0 0px rgba(0, 0, 0, 0.4);
  margin-left: 2px;
}
input[type="checkbox"].w2ui-toggle:disabled + div {
  opacity: 0.3;
}
/* Knob */
input[type="checkbox"].w2ui-toggle + div > div {
  float: left;
  width: 22px;
  height: 22px;
  border-radius: inherit;
  background: #f5f5f5;
  -webkit-transition-duration: 0.3s;
  -webkit-transition-property: transform, background-color, box-shadow;
  -moz-transition-duration: 0.3s;
  -moz-transition-property: transform, background-color;
  box-shadow: 0px 0px 1px #323232, 0 0 0 1px rgba(200, 200, 200, 0.6);
  pointer-events: none;
  margin-top: -1px;
  margin-left: -1px;
}
/* Default Green */
input[type="checkbox"].w2ui-toggle:checked + div {
  border: 1px solid #00a23f;
  box-shadow: inset 0 0 0 12px #54B350;
}
input[type="checkbox"].w2ui-toggle:checked + div > div {
  -webkit-transform: translate3d(24px, 0, 0);
  -moz-transform: translate3d(24px, 0, 0);
  background-color: #ffffff;
  box-shadow: 0px 2px 5px rgba(0, 0, 0, 0.3), 0 0 0 1px #00a23f;
}
/* Blue */
input[type="checkbox"].w2ui-toggle.blue:checked + div {
  border: 1px solid #206FAD;
  box-shadow: inset 0 0 0 12px #35A6EB;
}
input[type="checkbox"].w2ui-toggle.blue:checked + div > div {
  box-shadow: 0px 2px 5px rgba(0, 0, 0, 0.3), 0px 0px 0 1px #206fad;
}
input[type=checkbox].w2ui-toggle:focus {
  outline: none;
}
/*************************************************
* ---- Overlay and Bubble ----
*/
.w2ui-overlay {
  position: absolute;
  margin-top: 4px;
  margin-left: -17px;
  display: none;
  z-index: 1300;
  color: inherit;
  background-color: #fbfbfb;
  border-color: #fbfbfb;
  box-shadow: 0px 2px 10px #999999;
  border-radius: 4px;
  text-align: left;
}
.w2ui-overlay > div {
  border-radius: 4px;
  position: relative;
  border: 3px solid #777777;
}
.w2ui-overlay table td {
  color: inherit;
}
.w2ui-overlay:before {
  content: "";
  position: absolute;
  border-color: inherit;
}
.w2ui-overlay:after {
  content: "";
  position: absolute;
  border-color: inherit;
  bottom: 100%;
  left: 4px;
}
.w2ui-overlay.top-arrow:before {
  border-bottom: 12px solid #6f6f6f;
  border-right: 12px solid transparent;
  border-left: 12px solid transparent;
  bottom: 100%;
  margin-bottom: -3px;
}
.w2ui-overlay.top-arrow:after {
  border-bottom: 8px solid #000000;
  border-bottom-color: inherit;
  border-right: 8px solid transparent;
  border-left: 8px solid transparent;
  bottom: 100%;
  margin-bottom: -3px;
}
.w2ui-overlay.bottom-arrow:before {
  border-top: 12px solid #6f6f6f;
  border-right: 12px solid transparent;
  border-left: 12px solid transparent;
  top: 100%;
  margin-top: -3px;
}
.w2ui-overlay.bottom-arrow:after {
  border-top: 8px solid #000000;
  border-top-color: inherit;
  border-right: 8px solid transparent;
  border-left: 8px solid transparent;
  top: 100%;
  margin-top: -3px;
}
.w2ui-overlay.w2ui-overlay-popup {
  z-index: 1700;
}
.w2ui-overlay .w2ui-grid-searches {
  border-top: 3px solid #777777 !important;
}
.w2ui-overlay .no-color {
  border: 1px solid #eee;
  background: url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAMAAAAoLQ9TAAAABlBMVEX/////TgCFoIUYAAAAGUlEQVR42uXHIQEAAACDsNO/NJ4Kn9uC8wsJkAARUrXAjwAAAABJRU5ErkJggg==');
  background-size: 15px 15px;
}
.w2ui-tag {
  position: absolute;
  z-index: 1300;
  opacity: 0;
  -webkit-transition: opacity 0.3s;
  -moz-transition: opacity 0.3s;
  -ms-transition: opacity 0.3s;
  -o-transition: opacity 0.3s;
  transition: opacity 0.3s;
}
.w2ui-tag .w2ui-tag-body {
  box-sizing: border-box;
  display: inline-block;
  position: absolute;
  border-radius: 3px;
  padding: 6px 10px;
  margin-left: 10px;
  margin-top: 0px;
  background-color: rgba(60, 60, 60, 0.9);
  color: #ffffff !important;
  font-size: 11px;
  font-family: verdana;
  text-shadow: 1px 1px 3px #000;
  line-height: 1.4;
  letter-spacing: 0.1px;
}
.w2ui-tag .w2ui-tag-body.w2ui-tag-right:before {
  content: "";
  position: absolute;
  width: 0;
  height: 0;
  border-top: 6px solid transparent;
  border-right: 6px solid rgba(60, 60, 60, 0.9);
  border-bottom: 6px solid transparent;
  margin: 0px 0 0 -16px;
}
.w2ui-tag .w2ui-tag-body.w2ui-tag-left:after {
  content: "";
  position: absolute;
  top: 5px;
  width: 0;
  height: 0;
  border-top: 6px solid transparent;
  border-left: 6px solid rgba(60, 60, 60, 0.9);
  border-bottom: 6px solid transparent;
  margin: 0px 0 0 10px;
}
.w2ui-tag .w2ui-tag-body.w2ui-tag-bottom:before {
  content: "";
  position: absolute;
  width: 0;
  height: 0;
  border-right: 6px solid transparent;
  border-left: 6px solid transparent;
  border-bottom: 6px solid rgba(60, 60, 60, 0.9);
  margin: -12px 0 0 2px;
}
.w2ui-tag .w2ui-tag-body.w2ui-tag-top:after {
  content: "";
  position: absolute;
  left: 12px;
  bottom: -6px;
  width: 0;
  height: 0;
  border-right: 6px solid transparent;
  border-left: 6px solid transparent;
  border-top: 6px solid rgba(60, 60, 60, 0.9);
}
.w2ui-tag.w2ui-tag-popup {
  z-index: 1700;
}
/*
*    Drop down menu
*/
.w2ui-overlay table.w2ui-drop-menu {
  width: 100%;
  color: #000000;
  background-color: #ffffff;
  padding: 5px 0px;
  cursor: default;
}
.w2ui-overlay table.w2ui-drop-menu td {
  white-space: nowrap;
}
.w2ui-overlay table.w2ui-drop-menu .w2ui-item-even {
  color: inherit;
  background-color: #ffffff;
}
.w2ui-overlay table.w2ui-drop-menu .w2ui-item-odd {
  color: inherit;
  background-color: #f3f6fa;
}
.w2ui-overlay table.w2ui-drop-menu .w2ui-item-group {
  color: #444;
  font-weight: bold;
  background-color: #ECEDF0;
  border-bottom: 1px solid #D3D2D4;
}
.w2ui-overlay table.w2ui-drop-menu td.menu-icon {
  padding: 3px 0px 4px 6px;
  width: 20px;
}
.w2ui-overlay table.w2ui-drop-menu td.menu-icon > span {
  height: 20px;
  width: 18px;
}
.w2ui-overlay table.w2ui-drop-menu td.menu-text {
  padding: 8px 10px 8px 5px;
  width: auto;
}
.w2ui-overlay table.w2ui-drop-menu td.menu-count {
  text-align: right;
}
.w2ui-overlay table.w2ui-drop-menu td.menu-count > span {
  border: 1px solid #f6fcf4;
  border-radius: 20px;
  width: auto;
  height: 18px;
  padding: 2px 7px;
  margin: 3px 5px 0px 5px;
  background-color: #f2f8f0;
  color: #666666;
  box-shadow: 0px 0px 2px #474545;
  text-shadow: 1px 1px 0px #ffffff;
}
.w2ui-overlay table.w2ui-drop-menu td.menu-count > span.hotkey {
  border: none;
  border-radius: 0px;
  background-color: transparent !important;
  color: #888;
  box-shadow: none;
  text-shadow: none;
}
.w2ui-overlay table.w2ui-drop-menu tr:hover {
  color: inherit;
  background-color: #e6f0ff;
}
.w2ui-overlay table.w2ui-drop-menu tr.w2ui-selected {
  background-color: #b6d5fb;
}
.w2ui-overlay table.w2ui-drop-menu tr.w2ui-selected td {
  color: inherit;
}
.w2ui-overlay table.w2ui-drop-menu tr.w2ui-disabled {
  opacity: 0.4;
  background-color: white !important;
}
.w2ui-overlay table.w2ui-drop-menu .w2ui-icon {
  font-size: 14px;
  color: #8d99a7;
  display: inline-block;
  padding-top: 4px;
}
/*************************************************
* ---- Common Classes ----
*/
.w2ui-marker {
  color: #444;
  background-color: rgba(252, 244, 161, 0.48);
}
.w2ui-spinner {
  display: inline-block;
  background-size: 100%;
  background-repeat: no-repeat;
  background-image: url(data:image/gif;base64, );
}
/* common icons */
.w2ui-icon {
  background-repeat: no-repeat;
  height: 16px;
  width: 16px;
  overflow: hidden;
  margin: 2px 2px;
  display: inline-block;
}
.w2ui-icon.icon-search,
.w2ui-icon.icon-search-down {
  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACYAAAAgCAYAAAB+ZAqzAAACuElEQVRYw9WXSWhTQRjHR0UKLqhFaV0OUih68GAOWjyJKypCpAoV8aIiioIICiKiB1GMtE3MYmry2moXDz1UDx7sUXHBhQpSaRVxrYpWcMO9avx/8AJh/CbznHkxdeB3Cd/8589kvuUJkWcdjCTHghUgAi6DJ+AVeAqugSQIggniXywcNBJsB70g44EHYBcYXUhTM8EFj4ZkboKqQpiqAv2GprK8o7/f75t6pjn0M3gNPmri3vtycxAZA64qDvkJ2kENqAQTQQWoBg74qth3B4y3NbZDIX4fzNfsnQtuK/YfsjFVCh4pMq3Co0Y5uMVoUGkpy8aFT5xaeSzVEo45bXdBt4LeaLq1k0RXMYJfdDfFmAuAD4zWlty4UNyZEkm19MUb2zMw8Sfp1u+IWSrcIimLnTG8/SijdU6OO5poDESdtgHZVBzUHm/amhW7zoitMTS2mNHqASPk2FDCCcLMYK6p+obmulyxfiYLA4bGKFvfSnrUvkq5+Lpk8z4yRH8r3l/X4WiqJFfspSQ0CGYZGpsMnkt6L+h31Z76hpMdeOwPQ7H0NFnssST0C8wxNDaDKb6kP06150gsHahNNlVzYheZd7HJ0BiX4VRGhpmIhRixKyZilM2M1mnTArtIUbU3/qVO0H0GvmQ4CY4C3YopYYlHjXlggNG4R33Ypi2tVtwaPeTdNMkq9pVQZQdvFPs32zbx4aAjzxhDRfIAWAeWg7VgrzsY5ht/zoNJtubKwA3LITGjSKRyW3NTwaUCmKOSMd3WHH0ZJRQZZkOP1zFKZ3CB++4+aQ6kEeksWAb2a2L7qDv49S1Q6T72MOgEXa6RGFhP3wpS/B6NOWpRs0UxFg7eqTFHjX1hscxtAz/ymEuIYi0cvgF8Y0w5Ro3dZ3M1boJkTaXEUFlug6fsdsRQWzTj0cey+N/Xb2sj5lTh2M6OAAAAAElFTkSuQmCC) no-repeat center !important;
  background-size: 14px 12px !important;
  opacity: 0.9;
}
.w2ui-icon.icon-folder {
  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAGrSURBVDjLxZO7ihRBFIa/6u0ZW7GHBUV0UQQTZzd3QdhMQxOfwMRXEANBMNQX0MzAzFAwEzHwARbNFDdwEd31Mj3X7a6uOr9BtzNjYjKBJ6nicP7v3KqcJFaxhBVtZUAK8OHlld2st7Xl3DJPVONP+zEUV4HqL5UDYHr5xvuQAjgl/Qs7TzvOOVAjxjlC+ePSwe6DfbVegLVuT4r14eTr6zvA8xSAoBLzx6pvj4l+DZIezuVkG9fY2H7YRQIMZIBwycmzH1/s3F8AapfIPNF3kQk7+kw9PWBy+IZOdg5Ug3mkAATy/t0usovzGeCUWTjCz0B+Sj0ekfdvkZ3abBv+U4GaCtJ1iEm6ANQJ6fEzrG/engcKw/wXQvEKxSEKQxRGKE7Izt+DSiwBJMUSm71rguMYhQKrBygOIRStf4TiFFRBvbRGKiQLWP29yRSHKBTtfdBmHs0BUpgvtgF4yRFR+NUKi0XZcYjCeCG2smkzLAHkbRBmP0/Uk26O5YnUActBp1GsAI+S5nRJJJal5K1aAMrq0d6Tm9uI6zjyf75dAe6tx/SsWeD//o2/Ab6IH3/h25pOAAAAAElFTkSuQmCC) no-repeat center !important;
}
.w2ui-icon.icon-page {
  background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAINSURBVBgZBcG/r55zGAfg6/4+z3va01NHlYgzEfE7MdCIGISFgS4Gk8ViYyM2Mdlsko4GSf8Do0FLRCIkghhYJA3aVBtEz3nP89wf11VJvPDepdd390+8Nso5nESBQoq0pfvXm9fzWf19453LF85vASqJlz748vInb517dIw6EyYBIIG49u+xi9/c9MdvR//99MPPZ7+4cP4IZhhTPbwzT2d+vGoaVRRp1rRliVvHq+cfvM3TD82+7mun0o/ceO7NT+/4/KOXjwZU1ekk0840bAZzMQ2mooqh0A72d5x/6sB9D5zYnff3PoYBoWBgFKPKqDKqjCpjKr//dcu9p489dra88cydps30KswACfNEKanSaxhlntjJ8Mv12Paie+vZ+0+oeSwwQ0Iw1xAR1CiFNJkGO4wu3ZMY1AAzBI0qSgmCNJsJUEOtJSMaCTBDLyQ0CknAGOgyTyFFiLI2awMzdEcSQgSAAKVUmAeNkxvWJWCGtVlDmgYQ0GFtgg4pNtOwbBcwQy/Rife/2yrRRVI0qYCEBly8Z+P4qMEMy7JaVw72N568e+iwhrXoECQkfH91kY7jwwXMsBx1L93ZruqrK6uuiAIdSnTIKKPLPFcvay8ww/Hh+ufeznTXu49v95IMoQG3784gYXdTqvRmqn/Wpa/ADFX58MW3L71SVU9ETgEIQQQIOOzub+fhIvwPRDgeVjWDahIAAAAASUVORK5CYII=) no-repeat center !important;
}
/*************************************************
* ---- Locking portion of the screen (in grid, form, layout)
*/
.w2ui-lock {
  display: none;
  position: absolute;
  z-index: 1400;
  top: 0px;
  left: 0px;
  width: 100%;
  height: 100%;
  opacity: 0.15;
  filter: alpha(opacity=15);
  background-color: #333333;
}
.w2ui-lock-msg {
  display: none;
  position: absolute;
  z-index: 1400;
  top: 45%;
  left: 50%;
  -webkit-transform: translateX(-50%) translateY(-50%);
  -moz-transform: translateX(-50%) translateY(-50%);
  -ms-transform: translateX(-50%) translateY(-50%);
  -o-transform: translateX(-50%) translateY(-50%);
  transform: translateX(-50%) translateY(-50%);
  width: 200px;
  height: 80px;
  padding: 30px 8px;
  white-space: nowrap;
  text-overflow: ellipsis;
  overflow: hidden;
  font-size: 13px;
  font-family: Verdana, Arial, sans-serif;
  opacity: 0.8;
  filter: alpha(opacity=80);
  background-color: #555555;
  color: #ffffff;
  text-align: center;
  border-radius: 5px;
  border: 2px solid #444444;
}
.w2ui-lock-msg .w2ui-spinner {
  display: inline-block;
  width: 24px;
  height: 24px;
  margin: -3px 8px -7px -10px;
}
/*************************************************
* ---- Scroll contet, used in toolbar and tabs ----
*/
.w2ui-scroll-wrapper {
  overflow: hidden;
}
.w2ui-scroll-left,
.w2ui-scroll-right {
  top: 0;
  width: 18px;
  height: 100%;
  cursor: default !important;
  z-index: 10;
  display: none;
  position: absolute;
}
.w2ui-scroll-left:hover,
.w2ui-scroll-right:hover {
  background-color: #dddddd;
}
.w2ui-scroll-left {
  left: 0;
  box-shadow: 0px 0px 7px #5F5F5F;
  background: #f7f7f7 url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAQAAADZc7J/AAAAzklEQVR4Ae2THRDEMBCFzy1ucatb3eJ2uhi3uNUtbnGrW9zi1rOdNzdvdl7nDpvYt/9/r7+/51myZZf/zXkD2iMHHRSb0x3oskwMieK05PwEXqP4ExSL0wp0ROao2OOuMPOMdUL6XU1/oGLcFWb+NqyTd2W/P/2qTr9h+nFXhOkHXRHiNyjrgp/U/V+WaQcaNY13zZI0A1JvcVqAnrGDTdtDtZUHjHIJhxxVLN0iqXgCP1l/7h8U9kc6abyJ4/eNWPpGdBv+XdUK0K8cnvcBly2rDr7C1HQAAAAASUVORK5CYII=') center center no-repeat;
  background-size: 15px 12px;
}
.w2ui-scroll-right {
  right: 0;
  box-shadow: 0px 0px 7px #5F5F5F;
  background: #f7f7f7 url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAQAAADZc7J/AAAAz0lEQVR4Ae2UG7TGMBCEr1vd4la3uMUtuli3utWtbnGLW9zi9l/bDMzJG7u12cfJfLunf1+UEC9Bv0vVQwJ8hjRCaZafflb1C9RQf4OD0gSDE+i+PiJAabFhQc1y1AYYsJGLY3lgxM17uWPO56yPiFDqVPWgRtpIHSd1zPnwkBsdI58OlNwx4fP2X0TgfMTOoHSdKOXkpyNvEyQh7ul+4swxJSTQuwNDxz68l/ukVNbu0Neen5Z+KvzWxBAqHds349uPFJ/jVOrPjxUq++OLf+20q5+noXo0AAAAAElFTkSuQmCC') center center no-repeat;
  background-size: 15px 13px;
}
button.w2ui-btn {
  position: relative;
  display: inline-block;
  border-radius: 4px;
  margin: 0px 5px;
  padding: 6px 12px;
  color: #666;
  font-size: 12px;
  border: 1px solid #B6B6B6;
  background-image: -webkit-linear-gradient(#ffffff 0%, #e7e7e7 100%);
  background-image: -moz-linear-gradient(#ffffff 0%, #e7e7e7 100%);
  background-image: -ms-linear-gradient(#ffffff 0%, #e7e7e7 100%);
  background-image: -o-linear-gradient(#ffffff 0%, #e7e7e7 100%);
  background-image: linear-gradient(#ffffff 0%, #e7e7e7 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffe7e7e7', endColorstr='#ffffffff', GradientType=0);
  outline: none;
  box-shadow: 0px 1px 0px white;
  cursor: default;
  min-width: 75px;
  line-height: 110%;
  user-select: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  -webkit-tap-highlight-color: rgba(0, 0, 0, 0);
}
button.w2ui-btn:hover {
  text-decoration: none;
  border: 1px solid #bbb;
  background-image: -webkit-linear-gradient(#f7f7f7 0%, #dddddd 100%);
  background-image: -moz-linear-gradient(#f7f7f7 0%, #dddddd 100%);
  background-image: -ms-linear-gradient(#f7f7f7 0%, #dddddd 100%);
  background-image: -o-linear-gradient(#f7f7f7 0%, #dddddd 100%);
  background-image: linear-gradient(#f7f7f7 0%, #dddddd 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffdddddd', endColorstr='#fff7f7f7', GradientType=0);
  color: #333;
}
button.w2ui-btn:active,
button.w2ui-btn.clicked {
  border: 1px solid #999;
  background-image: -webkit-linear-gradient(#cccccc 0%, #cccccc 100%);
  background-image: -moz-linear-gradient(#cccccc 0%, #cccccc 100%);
  background-image: -ms-linear-gradient(#cccccc 0%, #cccccc 100%);
  background-image: -o-linear-gradient(#cccccc 0%, #cccccc 100%);
  background-image: linear-gradient(#cccccc 0%, #cccccc 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffcccccc', endColorstr='#ffcccccc', GradientType=0);
  text-shadow: 1px 1px 1px #eee;
}
button.w2ui-btn:disabled {
  border: 1px solid #bbb !important;
  background: #f7f7f7 !important;
  color: #bdbcbc !important;
  text-shadow: none !important;
}
button.w2ui-btn:focus:before {
  content: "";
  border: 1px dashed #aaa;
  border-radius: 3px;
  position: absolute;
  top: 2px;
  bottom: 2px;
  left: 2px;
  right: 2px;
  pointer-events: none;
}
button.w2ui-btn::-moz-focus-inner {
  border: 0;
}
button.w2ui-btn-blue {
  color: white;
  background-image: -webkit-linear-gradient(#80c0f7 0%, #269df0 100%);
  background-image: -moz-linear-gradient(#80c0f7 0%, #269df0 100%);
  background-image: -ms-linear-gradient(#80c0f7 0%, #269df0 100%);
  background-image: -o-linear-gradient(#80c0f7 0%, #269df0 100%);
  background-image: linear-gradient(#80c0f7 0%, #269df0 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ff269df0', endColorstr='#ff80c0f7', GradientType=0);
  border: 1px solid #538AB7;
  text-shadow: 0px 0px 1px #111111;
}
button.w2ui-btn-blue:hover {
  color: white;
  background-image: -webkit-linear-gradient(#73b6f0 0%, #2391dd 100%);
  background-image: -moz-linear-gradient(#73b6f0 0%, #2391dd 100%);
  background-image: -ms-linear-gradient(#73b6f0 0%, #2391dd 100%);
  background-image: -o-linear-gradient(#73b6f0 0%, #2391dd 100%);
  background-image: linear-gradient(#73b6f0 0%, #2391dd 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ff2391dd', endColorstr='#ff73b6f0', GradientType=0);
  border: 1px solid #497BA3;
  text-shadow: 0px 0px 1px #111111;
}
button.w2ui-btn-blue:active,
button.w2ui-btn-blue.clicked {
  color: white;
  background-image: -webkit-linear-gradient(#1e83c9 0%, #1e83c9 100%);
  background-image: -moz-linear-gradient(#1e83c9 0%, #1e83c9 100%);
  background-image: -ms-linear-gradient(#1e83c9 0%, #1e83c9 100%);
  background-image: -o-linear-gradient(#1e83c9 0%, #1e83c9 100%);
  background-image: linear-gradient(#1e83c9 0%, #1e83c9 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ff1e83c9', endColorstr='#ff1e83c9', GradientType=0);
  border: 1px solid #1268A6;
  text-shadow: 0px 0px 1px #111111;
}
button.w2ui-btn-blue:focus:before {
  border: 1px dashed #e8e8e8;
}
button.w2ui-btn-green {
  color: white;
  background-image: -webkit-linear-gradient(#81cf81 0%, #52a452 100%);
  background-image: -moz-linear-gradient(#81cf81 0%, #52a452 100%);
  background-image: -ms-linear-gradient(#81cf81 0%, #52a452 100%);
  background-image: -o-linear-gradient(#81cf81 0%, #52a452 100%);
  background-image: linear-gradient(#81cf81 0%, #52a452 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ff52a452', endColorstr='#ff81cf81', GradientType=0);
  border: 1px solid #479247;
  text-shadow: 0px 0px 1px #111111;
}
button.w2ui-btn-green:hover {
  color: white;
  background-image: -webkit-linear-gradient(#6abe68 0%, #3f8f3d 100%);
  background-image: -moz-linear-gradient(#6abe68 0%, #3f8f3d 100%);
  background-image: -ms-linear-gradient(#6abe68 0%, #3f8f3d 100%);
  background-image: -o-linear-gradient(#6abe68 0%, #3f8f3d 100%);
  background-image: linear-gradient(#6abe68 0%, #3f8f3d 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ff3f8f3d', endColorstr='#ff6abe68', GradientType=0);
  border: 1px solid #479247;
  text-shadow: 0px 0px 1px #111111;
}
button.w2ui-btn-green:active,
button.w2ui-btn-green.clicked {
  color: white;
  background-image: -webkit-linear-gradient(#377d36 0%, #377d36 100%);
  background-image: -moz-linear-gradient(#377d36 0%, #377d36 100%);
  background-image: -ms-linear-gradient(#377d36 0%, #377d36 100%);
  background-image: -o-linear-gradient(#377d36 0%, #377d36 100%);
  background-image: linear-gradient(#377d36 0%, #377d36 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ff377d36', endColorstr='#ff377d36', GradientType=0);
  border: 1px solid #555 !important;
  text-shadow: 0px 0px 1px #111111;
}
button.w2ui-btn-green:focus:before {
  border: 1px dashed #e8e8e8;
}
button.w2ui-btn-orange {
  color: white;
  background-image: -webkit-linear-gradient(#fcc272 0%, #fb8822 100%);
  background-image: -moz-linear-gradient(#fcc272 0%, #fb8822 100%);
  background-image: -ms-linear-gradient(#fcc272 0%, #fb8822 100%);
  background-image: -o-linear-gradient(#fcc272 0%, #fb8822 100%);
  background-image: linear-gradient(#fcc272 0%, #fb8822 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#fffb8822', endColorstr='#fffcc272', GradientType=0);
  border: 1px solid #B68B4C;
  text-shadow: 0px 0px 1px #111111;
}
button.w2ui-btn-orange:hover {
  color: white;
  background-image: -webkit-linear-gradient(#f4ad59 0%, #f1731f 100%);
  background-image: -moz-linear-gradient(#f4ad59 0%, #f1731f 100%);
  background-image: -ms-linear-gradient(#f4ad59 0%, #f1731f 100%);
  background-image: -o-linear-gradient(#f4ad59 0%, #f1731f 100%);
  background-image: linear-gradient(#f4ad59 0%, #f1731f 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#fff1731f', endColorstr='#fff4ad59', GradientType=0);
  border: 1px solid #B68B4C;
  text-shadow: 0px 0px 1px #111111;
}
button.w2ui-btn-orange:active,
button.w2ui-btn-orange.clicked {
  color: white;
  border: 1px solid #666;
  background-image: -webkit-linear-gradient(#b98747 0%, #b98747 100%);
  background-image: -moz-linear-gradient(#b98747 0%, #b98747 100%);
  background-image: -ms-linear-gradient(#b98747 0%, #b98747 100%);
  background-image: -o-linear-gradient(#b98747 0%, #b98747 100%);
  background-image: linear-gradient(#b98747 0%, #b98747 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffb98747', endColorstr='#ffb98747', GradientType=0);
  text-shadow: 0px 0px 1px #111111;
}
button.w2ui-btn-orange:focus:before {
  border: 1px dashed #f9f9f9;
}
button.w2ui-btn-red {
  color: white;
  background-image: -webkit-linear-gradient(#ff6e70 0%, #c72d2d 100%);
  background-image: -moz-linear-gradient(#ff6e70 0%, #c72d2d 100%);
  background-image: -ms-linear-gradient(#ff6e70 0%, #c72d2d 100%);
  background-image: -o-linear-gradient(#ff6e70 0%, #c72d2d 100%);
  background-image: linear-gradient(#ff6e70 0%, #c72d2d 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffc72d2d', endColorstr='#ffff6e70', GradientType=0);
  border: 1px solid #BB3C3E;
  text-shadow: 0px 0px 1px #111111;
}
button.w2ui-btn-red:hover {
  color: white;
  background-image: -webkit-linear-gradient(#ee696c 0%, #ae2527 100%);
  background-image: -moz-linear-gradient(#ee696c 0%, #ae2527 100%);
  background-image: -ms-linear-gradient(#ee696c 0%, #ae2527 100%);
  background-image: -o-linear-gradient(#ee696c 0%, #ae2527 100%);
  background-image: linear-gradient(#ee696c 0%, #ae2527 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffae2527', endColorstr='#ffee696c', GradientType=0);
  border: 1px solid #BB3C3E;
  text-shadow: 0px 0px 1px #111111;
}
button.w2ui-btn-red:active,
button.w2ui-btn-red.clicked {
  color: white;
  border: 1px solid #861C1E;
  background-image: -webkit-linear-gradient(#9c2123 0%, #9c2123 100%);
  background-image: -moz-linear-gradient(#9c2123 0%, #9c2123 100%);
  background-image: -ms-linear-gradient(#9c2123 0%, #9c2123 100%);
  background-image: -o-linear-gradient(#9c2123 0%, #9c2123 100%);
  background-image: linear-gradient(#9c2123 0%, #9c2123 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ff9c2123', endColorstr='#ff9c2123', GradientType=0);
  text-shadow: 0px 0px 1px #111111;
}
button.w2ui-btn-red:focus:before {
  border: 1px dashed #ddd;
}
button.w2ui-btn-small {
  padding: 5px !important;
  margin: 0px;
  min-width: 0px;
}
button.w2ui-btn-small:focus:before {
  border-radius: 2px;
  top: 2px;
  bottom: 2px;
  left: 2px;
  right: 2px;
}
/*************************************************
* ---- Forms ----
*/
.w2ui-form {
  position: relative;
  color: #000000;
  background-color: #f2f2f2;
  border: 1px solid #c0c0c0;
  border-radius: 3px;
  padding: 0px;
  overflow: hidden !important;
}
.w2ui-form > div {
  position: absolute;
  overflow: hidden;
}
.w2ui-form .w2ui-form-header {
  position: absolute;
  left: 0;
  right: 0;
  border-bottom: 1px solid #99bbe8 !important;
  overflow: hidden;
  color: #444444;
  font-size: 13px;
  text-align: center;
  padding: 8px;
  background-image: -webkit-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -moz-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -ms-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -o-linear-gradient(#dae6f3, #c2d5ed);
  background-image: linear-gradient(#dae6f3, #c2d5ed);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffdae6f3', endColorstr='#ffc2d5ed', GradientType=0);
  border-top-left-radius: 3px;
  border-top-right-radius: 3px;
}
.w2ui-form .w2ui-form-toolbar {
  position: absolute;
  left: 0px;
  right: 0px;
  margin: 0px;
  padding: 6px 3px;
  border-bottom: 1px solid #d5d8d8;
}
.w2ui-form .w2ui-form-tabs {
  margin: 0px;
  padding: 0px;
}
.w2ui-form .w2ui-tabs {
  position: absolute;
  left: 0;
  right: 0;
  border-top-left-radius: 3px;
  border-top-right-radius: 3px;
  padding-top: 5px !important;
  background-color: #fafafa;
}
.w2ui-form .w2ui-tabs .w2ui-tab.active {
  background-color: #f2f2f2;
}
.w2ui-form .w2ui-page {
  position: absolute;
  left: 0;
  right: 0;
  overflow: auto;
  padding: 10px;
  border-left: 1px solid inherit;
  border-right: 1px solid inherit;
  background-color: inherit;
  border-radius: 3px;
}
.w2ui-form .w2ui-buttons {
  position: absolute;
  left: 0;
  right: 0;
  bottom: 0;
  text-align: center;
  border-top: 1px solid #d5d8d8;
  border-bottom: 0px solid #d5d8d8;
  background-color: #fafafa;
  padding: 15px 0px !important;
  border-bottom-left-radius: 3px;
  border-bottom-right-radius: 3px;
}
.w2ui-form .w2ui-buttons input[type="button"],
.w2ui-form .w2ui-buttons button {
  min-width: 80px;
  margin-right: 5px;
}
.w2ui-form input[type=checkbox],
.w2ui-form input[type=radio] {
  margin-top: 4px;
  margin-bottom: 4px;
}
.w2ui-form input[type=checkbox].w2ui-toggle {
  margin: 0px;
}
.w2ui-group-title {
  padding: 5px 2px;
  color: #8D96A2;
  text-shadow: 1px 1px 2px #fdfdfd;
  font-size: 120%;
}
.w2ui-group {
  background-color: #f9f9f9;
  margin: 5px 0px 10px 0px;
  padding: 10px 5px;
  border-top: 1px solid #dcdcdc;
  border-bottom: 1px solid #dcdcdc;
}
.w2ui-field > label {
  display: block;
  float: left;
  margin-top: 7px;
  margin-bottom: 3px;
  width: 120px;
  padding: 0px;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
  text-align: right;
  min-height: 20px;
  color: #666;
}
.w2ui-field > div {
  /* do not include width */
  margin-bottom: 3px;
  margin-left: 128px;
  padding: 3px;
  min-height: 28px;
  float: none;
}
.w2ui-field.w2ui-required > div {
  position: relative;
}
.w2ui-field.w2ui-required > div::before {
  content: '*';
  position: absolute;
  margin-top: 5px;
  margin-left: -9px;
  color: #ff0000;
}
.w2ui-field.w2ui-span1 > label {
  width: 20px;
}
.w2ui-field.w2ui-span1 > div {
  margin-left: 28px;
}
.w2ui-field.w2ui-span2 > label {
  width: 40px;
}
.w2ui-field.w2ui-span2 > div {
  margin-left: 48px;
}
.w2ui-field.w2ui-span3 > label {
  width: 60px;
}
.w2ui-field.w2ui-span3 > div {
  margin-left: 68px;
}
.w2ui-field.w2ui-span4 > label {
  width: 80px;
}
.w2ui-field.w2ui-span4 > div {
  margin-left: 88px;
}
.w2ui-field.w2ui-span5 > label {
  width: 100px;
}
.w2ui-field.w2ui-span5 > div {
  margin-left: 108px;
}
.w2ui-field.w2ui-span6 > label {
  width: 120px;
}
.w2ui-field.w2ui-span6 > div {
  margin-left: 128px;
}
.w2ui-field.w2ui-span7 > label {
  width: 140px;
}
.w2ui-field.w2ui-span7 > div {
  margin-left: 148px;
}
.w2ui-field.w2ui-span8 > label {
  width: 160px;
}
.w2ui-field.w2ui-span8 > div {
  margin-left: 168px;
}
.w2ui-field.w2ui-span9 > label {
  width: 180px;
}
.w2ui-field.w2ui-span9 > div {
  margin-left: 188px;
}
.w2ui-field.w2ui-span10 > label {
  width: 200px;
}
.w2ui-field.w2ui-span10 > div {
  margin-left: 208px;
}
.w2ui-error {
  border: 1px solid #ffa8a8 !important;
  background-color: #fff4eb !important;
}
.w2field {
  padding: 3px;
  border-radius: 3px;
  border: 1px solid silver;
}
.w2ui-field-helper {
  position: absolute;
  display: inline-block;
  line-height: 100%;
  /* pointer-events: none; - do not use as IE does not support it */
  user-select: none;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
}
.w2ui-field-helper .w2ui-field-up {
  position: absolute;
  top: 0px;
  padding: 2px 3px;
}
.w2ui-field-helper .w2ui-field-down {
  position: absolute;
  bottom: 0px;
  padding: 2px 3px;
}
.w2ui-field-helper .arrow-up:hover {
  border-bottom-color: #81C6FF;
}
.w2ui-field-helper .arrow-down:hover {
  border-top-color: #81C6FF;
}
/*
*    ARROWS
*/
.arrow-up {
  background: none;
  width: 0;
  height: 0;
  border-left: 4px solid transparent;
  /* left arrow slant */
  border-right: 4px solid transparent;
  /* right arrow slant */
  border-bottom: 5px solid #777;
  /* bottom, add background color here */
  font-size: 0;
  line-height: 0;
}
.arrow-down {
  background: none;
  width: 0;
  height: 0;
  border-left: 4px solid transparent;
  border-right: 4px solid transparent;
  border-top: 5px solid #777;
  font-size: 0;
  line-height: 0;
}
.arrow-left {
  background: none;
  width: 0;
  height: 0;
  border-bottom: 4px solid transparent;
  /* left arrow slant */
  border-top: 4px solid transparent;
  /* right arrow slant */
  border-right: 5px solid #777;
  /* bottom, add background color here */
  font-size: 0;
  line-height: 0;
}
.arrow-right {
  background: none;
  width: 0;
  height: 0;
  border-bottom: 4px solid transparent;
  /* left arrow slant */
  border-top: 4px solid transparent;
  /* right arrow slant */
  border-left: 5px solid #777;
  /* bottom, add background color here */
  font-size: 0;
  line-height: 0;
}
/*
*    COLOR overlay
*/
.w2ui-color {
  padding: 5px;
  padding-top: 8px;
  background-color: white;
  border-radius: 3px;
}
.w2ui-color > table {
  table-layout: fixed;
  width: 160px;
}
.w2ui-color > table td {
  width: 20px;
  height: 20px;
  text-align: center;
}
.w2ui-color > table td div {
  cursor: pointer;
  display: inline-block;
  width: 16px;
  height: 17px;
  padding: 1px 4px;
  border: 1px solid transparent;
  color: white;
  text-shadow: 0px 0px 2px #000;
}
.w2ui-color > table td div:hover {
  outline: 1px solid #666;
  border: 1px solid #fff;
}
/*
*    DATE overlay
*/
.w2ui-calendar {
  margin: 0px;
  padding: 1px;
  line-height: 108%;
}
.w2ui-calendar .w2ui-calendar-title {
  margin: 0px -1px;
  padding: 7px 2px;
  background-image: -webkit-linear-gradient(#f6f6f6, #d9d9d9);
  background-image: -moz-linear-gradient(#f6f6f6, #d9d9d9);
  background-image: -ms-linear-gradient(#f6f6f6, #d9d9d9);
  background-image: -o-linear-gradient(#f6f6f6, #d9d9d9);
  background-image: linear-gradient(#f6f6f6, #d9d9d9);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#fff6f6f6', endColorstr='#ffd9d9d9', GradientType=0);
  border-bottom: 1px solid #bbbbbb;
  color: #555555;
  text-align: center;
  text-shadow: 1px 1px 1px #eeeeee;
  cursor: pointer;
}
.w2ui-calendar .w2ui-calendar-jump {
  position: absolute;
  top: 27px;
  left: 0px;
  right: 0px;
  bottom: 0px;
  background-color: #FaFaFa;
}
.w2ui-calendar .w2ui-calendar-jump > :first-child {
  position: absolute;
  top: 0px;
  left: 0px;
  bottom: 0px;
  width: 110px;
  overflow: hidden;
  padding-top: 5px;
  border-right: 1px solid #c0c0c0;
}
.w2ui-calendar .w2ui-calendar-jump > :last-child {
  position: absolute;
  top: 0px;
  right: 0px;
  bottom: 0px;
  width: 88px;
  overflow-x: hidden;
  overflow-y: auto;
  padding-top: 5px;
  text-align: center;
}
.w2ui-calendar .w2ui-calendar-jump .w2ui-jump-month,
.w2ui-calendar .w2ui-calendar-jump .w2ui-jump-year {
  display: inline-block;
  padding: 5px 0px;
  text-align: center;
  float: left;
  margin: 2px;
  width: 50px;
  cursor: default;
  border: 1px solid transparent;
  border-radius: 2px;
}
.w2ui-calendar .w2ui-calendar-jump .w2ui-jump-year {
  float: none;
  width: 95%;
}
.w2ui-calendar .w2ui-calendar-jump .w2ui-jump-month:hover,
.w2ui-calendar .w2ui-calendar-jump .w2ui-jump-year:hover {
  border: 1px solid #cccccc;
  color: #000000;
  background-color: #efefef;
}
.w2ui-calendar .w2ui-calendar-jump .w2ui-jump-month.selected,
.w2ui-calendar .w2ui-calendar-jump .w2ui-jump-year.selected {
  border: 1px solid #cccccc;
  color: #000000;
  background-color: #dadada;
}
.w2ui-calendar .w2ui-calendar-previous,
.w2ui-calendar .w2ui-calendar-next {
  width: 24px;
  height: 20px;
  color: #666666;
  border: 1px solid transparent;
  border-radius: 3px;
  padding: 2px 3px 1px 2px;
  margin: -4px 0px 0px 0px;
  cursor: default;
}
.w2ui-calendar .w2ui-calendar-previous:hover,
.w2ui-calendar .w2ui-calendar-next:hover {
  border: 1px solid #c0c0c0;
  background-color: #efefef;
}
.w2ui-calendar .w2ui-calendar-previous > div,
.w2ui-calendar .w2ui-calendar-next > div {
  position: absolute;
  border-left: 4px solid #888;
  border-top: 4px solid #888;
  border-right: 4px solid transparent;
  border-bottom: 4px solid transparent;
  width: 0px;
  height: 0px;
  padding: 0px;
  margin: 3px 0px 0px 0px;
}
.w2ui-calendar .w2ui-calendar-previous {
  float: left;
}
.w2ui-calendar .w2ui-calendar-previous > div {
  -webkit-transform: rotate(-45deg);
  -moz-transform: rotate(-45deg);
  -ms-transform: rotate(-45deg);
  -o-transform: rotate(-45deg);
  transform: rotate(-45deg);
  margin-left: 6px;
}
.w2ui-calendar .w2ui-calendar-next {
  float: right;
}
.w2ui-calendar .w2ui-calendar-next > div {
  -webkit-transform: rotate(135deg);
  -moz-transform: rotate(135deg);
  -ms-transform: rotate(135deg);
  -o-transform: rotate(135deg);
  transform: rotate(135deg);
  margin-left: 2px;
  margin-right: 2px;
}
.w2ui-calendar .w2ui-calendar-now {
  cursor: pointer;
  margin-bottom: 10px;
  text-align: center;
}
.w2ui-calendar .w2ui-calendar-now:hover {
  color: #0A96DE;
}
.w2ui-calendar table.w2ui-calendar-days {
  padding: 0px;
}
.w2ui-calendar table.w2ui-calendar-days td {
  border: 1px solid #ffffff;
  color: #000000;
  background-color: #f9f9f9;
  padding: 6px;
  cursor: default;
  text-align: right;
}
.w2ui-calendar table.w2ui-calendar-days td.w2ui-saturday,
.w2ui-calendar table.w2ui-calendar-days td.w2ui-sunday {
  border: 1px solid #ffffff;
  color: #c8493b;
  background-color: #f9f9f9;
}
.w2ui-calendar table.w2ui-calendar-days td.w2ui-saturday:hover,
.w2ui-calendar table.w2ui-calendar-days td.w2ui-sunday:hover {
  border: 1px solid #cccccc;
  color: #000000;
  background-color: #e9e9e9;
}
.w2ui-calendar table.w2ui-calendar-days td.w2ui-saturday.w2ui-blocked,
.w2ui-calendar table.w2ui-calendar-days td.w2ui-sunday.w2ui-blocked {
  text-decoration: line-through;
  border: 1px solid #ffffff;
  color: #cccccc;
  background-color: #ffffff;
}
.w2ui-calendar table.w2ui-calendar-days td.w2ui-today {
  color: #000000;
  background-color: #e2f7cd;
}
.w2ui-calendar table.w2ui-calendar-days td:hover {
  border: 1px solid #cccccc;
  color: #000000;
  background-color: #e9e9e9;
}
.w2ui-calendar table.w2ui-calendar-days td.w2ui-date-selected {
  border: 1px solid #8cb067;
}
.w2ui-calendar table.w2ui-calendar-days td.w2ui-blocked {
  text-decoration: line-through;
  border: 1px solid #ffffff;
  color: #cccccc;
  background-color: #ffffff;
}
.w2ui-calendar table.w2ui-calendar-days td.w2ui-day-empty {
  border: 1px solid #ffffff;
  background-color: #fdfdfd;
}
.w2ui-calendar table.w2ui-calendar-days tr.w2ui-day-title td {
  border: 1px solid #ffffff;
  color: #808080;
  background-color: #ffffff;
  text-align: center;
  padding: 6px;
}
/*
*    Time
*/
.w2ui-calendar-time {
  padding: 5px;
  cursor: default;
}
.w2ui-calendar-time td div {
  padding: 7px 10px;
  text-align: center;
  border: 1px solid transparent;
  white-space: nowrap;
}
.w2ui-calendar-time td:nth-child(even) {
  background-color: #f6f6f6;
}
.w2ui-calendar-time td div:hover {
  border: 1px solid #cccccc;
  color: #000000;
  background-color: #e9e9e9;
}
.w2ui-calendar-time td div.w2ui-blocked {
  text-decoration: line-through;
  border: 1px solid #ffffff;
  color: #cccccc;
  background-color: #ffffff;
}
.w2ui-select {
  cursor: default;
  color: black !important;
  background-image: url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAALCAQAAACnzwd+AAAAcklEQVR4AcXMsQFBQQDG4P9tAgC0gJYRQJZgKQMwCqCku6vVAAAA+NJHP4KHOk0aV2pRw61n4BBmyOxKQ8I4ehZeuhd3HTx6DQEGZ7sBfr2OOOOj3Yi43kMKs9sZknofOexqZ8npMygwWZTX51CipP+YA1OiZJbYYg9lAAAAAElFTkSuQmCC'), -webkit-linear-gradient(top, #ffffff 20%, #f6f6f6 50%, #eeeeee 52%, #f4f4f4 100%);
  background-image: url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAALCAQAAACnzwd+AAAAcklEQVR4AcXMsQFBQQDG4P9tAgC0gJYRQJZgKQMwCqCku6vVAAAA+NJHP4KHOk0aV2pRw61n4BBmyOxKQ8I4ehZeuhd3HTx6DQEGZ7sBfr2OOOOj3Yi43kMKs9sZknofOexqZ8npMygwWZTX51CipP+YA1OiZJbYYg9lAAAAAElFTkSuQmCC'), -moz-linear-gradient(top, #ffffff 20%, #f6f6f6 50%, #eeeeee 52%, #f4f4f4 100%);
  background-image: url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAALCAQAAACnzwd+AAAAcklEQVR4AcXMsQFBQQDG4P9tAgC0gJYRQJZgKQMwCqCku6vVAAAA+NJHP4KHOk0aV2pRw61n4BBmyOxKQ8I4ehZeuhd3HTx6DQEGZ7sBfr2OOOOj3Yi43kMKs9sZknofOexqZ8npMygwWZTX51CipP+YA1OiZJbYYg9lAAAAAElFTkSuQmCC'), -ms-linear-gradient(top, #ffffff 20%, #f6f6f6 50%, #eeeeee 52%, #f4f4f4 100%);
  background-image: url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAALCAQAAACnzwd+AAAAcklEQVR4AcXMsQFBQQDG4P9tAgC0gJYRQJZgKQMwCqCku6vVAAAA+NJHP4KHOk0aV2pRw61n4BBmyOxKQ8I4ehZeuhd3HTx6DQEGZ7sBfr2OOOOj3Yi43kMKs9sZknofOexqZ8npMygwWZTX51CipP+YA1OiZJbYYg9lAAAAAElFTkSuQmCC'), -o-linear-gradient(top, #ffffff 20%, #f6f6f6 50%, #eeeeee 52%, #f4f4f4 100%);
  background-image: url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAALCAQAAACnzwd+AAAAcklEQVR4AcXMsQFBQQDG4P9tAgC0gJYRQJZgKQMwCqCku6vVAAAA+NJHP4KHOk0aV2pRw61n4BBmyOxKQ8I4ehZeuhd3HTx6DQEGZ7sBfr2OOOOj3Yi43kMKs9sZknofOexqZ8npMygwWZTX51CipP+YA1OiZJbYYg9lAAAAAElFTkSuQmCC'), linear-gradient(top, #ffffff 20%, #f6f6f6 50%, #eeeeee 52%, #f4f4f4 100%);
  background-size: 17px 6px, 100% 100%;
  background-position: right center, left top;
  background-repeat: no-repeat, no-repeat;
}
.w2ui-select[readonly],
.w2ui-select[disabled] {
  background-image: none;
  background-color: #f1f1f1 !important;
  color: #777 !important;
}
/*
*    ENUM items
*/
.w2ui-list {
  color: inherit;
  position: absolute;
  padding: 0px;
  margin: 0px;
  min-height: 25px;
  overflow: auto;
  border: 1px solid #c0c0c0;
  border-radius: 3px;
  font-size: 6px;
  line-height: 100%;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
  background-color: #ffffff;
}
.w2ui-list input[type=text] {
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  -ms-box-shadow: none;
  -o-box-shadow: none;
  box-shadow: none;
}
.w2ui-list ul {
  list-style-type: none;
  background-color: black;
  margin: 0px;
  padding: 0px;
}
.w2ui-list ul li {
  float: left;
  margin: 2px 1px 0px 2px;
  border-radius: 3px;
  width: auto;
  padding: 3px 10px 1px 7px;
  border: 1px solid #88b0d6;
  background-color: #eff3f5;
  white-space: nowrap;
  cursor: default;
  font-family: verdana;
  font-size: 11px;
  line-height: 100%;
  height: 20px;
  overflow: hidden;
  text-overflow: ellipsis;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-list ul li:hover {
  background-color: #d0dbe1;
}
.w2ui-list ul li:last-child {
  border-radius: 0px;
  border: 1px solid transparent;
  background-color: transparent;
}
.w2ui-list ul li:last-child input {
  padding: 1px;
  padding-top: 0px;
  margin: 0px;
  border: 0px;
  outline: none;
  height: auto;
  line-height: 100%;
  font-size: inherit;
  font-family: inherit;
  background-color: transparent;
}
.w2ui-list ul li .w2ui-list-remove {
  float: right;
  width: 15px;
  height: 14px;
  margin: -1px -9px 0px 3px;
  border-radius: 15px;
}
.w2ui-list ul li .w2ui-list-remove:hover {
  background-color: #D77F7F;
  color: white;
}
.w2ui-list ul li .w2ui-list-remove:before {
  position: relative;
  top: 0px;
  padding: 0px;
  margin: 0px;
  left: 5px;
  color: inherit;
  opacity: 0.7;
  text-shadow: inherit;
  font-size: inherit;
  font-variant: small-caps;
  content: 'x';
  line-height: 100%;
}
.w2ui-list ul li > span.file-size {
  pointer-events: none;
  color: #777;
}
.w2ui-list .w2ui-enum-placeholder {
  display: inline;
  position: absolute;
  pointer-events: none;
  color: #999;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-list.w2ui-file-dragover {
  background-color: #E4FFDA;
  border: 1px solid #93E07D;
}
/*************************************************
* ---- Layout ----
*/
.w2ui-layout {
  overflow: hidden !important;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-layout * {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-layout > div {
  position: absolute;
  overflow: hidden;
  border: 0px;
  margin: 0px;
  padding: 0px;
  outline: 0px;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-layout > div .w2ui-panel {
  display: none;
  position: absolute;
  z-index: 120;
}
.w2ui-layout > div .w2ui-panel .w2ui-panel-title {
  position: absolute;
  left: 0px;
  top: 0px;
  right: 0px;
  padding: 5px;
  background-image: -webkit-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -moz-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -ms-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -o-linear-gradient(#dae6f3, #c2d5ed);
  background-image: linear-gradient(#dae6f3, #c2d5ed);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffdae6f3', endColorstr='#ffc2d5ed', GradientType=0);
  border: 1px solid #b9cee9;
  border-bottom: 1px solid #99bbe8;
}
.w2ui-layout > div .w2ui-panel .w2ui-panel-tabs {
  position: absolute;
  left: 0px;
  top: 0px;
  right: 0px;
  z-index: 2;
  display: none;
  overflow: hidden;
  background-color: #fafafa;
  padding: 4px 0px;
}
.w2ui-layout > div .w2ui-panel .w2ui-panel-tabs > .w2ui-tab.active {
  background-color: #f2f2f2;
}
.w2ui-layout > div .w2ui-panel .w2ui-panel-toolbar {
  position: absolute;
  left: 0px;
  top: 0px;
  right: 0px;
  z-index: 2;
  display: none;
  overflow: hidden;
  background-color: #fafafa;
  border-bottom: 1px solid silver;
  padding: 4px;
}
.w2ui-layout > div .w2ui-panel .w2ui-panel-content {
  position: absolute;
  left: 0px;
  top: 0px;
  right: 0px;
  bottom: 0px;
  z-index: 1;
  color: inherit;
  background-color: #f2f2f2;
}
.w2ui-layout > div .w2ui-resizer {
  display: none;
  position: absolute;
  z-index: 121;
  background-color: transparent;
}
.w2ui-layout > div .w2ui-resizer:hover,
.w2ui-layout > div .w2ui-resizer.active {
  background-color: #c8cad1;
}
/*************************************************
* ---- Grid ----
*/
.w2ui-grid {
  position: relative;
  border: 1px solid #c0c0c0;
  border-radius: 2px;
  overflow: hidden !important;
}
.w2ui-grid > div {
  position: absolute;
  overflow: hidden;
}
.w2ui-grid .w2ui-grid-header {
  position: absolute;
  border-bottom: 1px solid #99bbe8 !important;
  height: 28px;
  overflow: hidden;
  color: #444444;
  font-size: 13px;
  text-align: center;
  padding: 7px;
  background-image: -webkit-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -moz-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -ms-linear-gradient(#dae6f3, #c2d5ed);
  background-image: -o-linear-gradient(#dae6f3, #c2d5ed);
  background-image: linear-gradient(#dae6f3, #c2d5ed);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffdae6f3', endColorstr='#ffc2d5ed', GradientType=0);
  border-top-left-radius: 2px;
  border-top-right-radius: 2px;
}
.w2ui-grid .w2ui-grid-toolbar {
  position: absolute;
  border-bottom: 1px solid #c0c0c0;
  background-color: #eaeaea;
  height: 38px;
  padding: 6px 1px 4px 1px;
  margin: 0px;
  box-shadow: 0px 1px 2px #dddddd;
}
.w2ui-grid .w2ui-toolbar-search {
  width: 160px;
  margin-right: 3px;
}
.w2ui-grid .w2ui-toolbar-search .w2ui-search-all {
  outline: none !important;
  width: 160px !important;
  border-radius: 3px !important;
  line-height: normal !important;
  height: 23px !important;
  border: 1px solid #b9b9b9 !important;
  color: #000000 !important;
  background-color: #ffffff !important;
  padding: 3px 18px 3px 21px !important;
  margin: 0px !important;
  margin-top: 1px !important;
}
.w2ui-grid .w2ui-toolbar-search .w2ui-search-down {
  position: absolute;
  margin-top: -7px;
  margin-left: 4px;
}
.w2ui-grid .w2ui-toolbar-search .w2ui-search-clear {
  position: absolute;
  width: 16px;
  height: 16px;
  margin-top: -8px;
  margin-left: -20px;
  border-radius: 15px;
  cursor: default;
}
.w2ui-grid .w2ui-toolbar-search .w2ui-search-clear:hover {
  background-color: #D77F7F;
  color: white;
}
.w2ui-grid .w2ui-toolbar-search .w2ui-search-clear:before {
  position: relative;
  top: 2px;
  left: 5px;
  opacity: 0.6;
  color: inherit;
  text-shadow: inherit;
  content: 'x';
  cursor: default;
}
.w2ui-grid .w2ui-grid-body {
  position: absolute;
  overflow: hidden;
  padding: 0px;
  background-color: #ffffff;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
}
.w2ui-grid .w2ui-grid-body input,
.w2ui-grid .w2ui-grid-body select,
.w2ui-grid .w2ui-grid-body textarea {
  user-select: text;
  -webkit-user-select: text;
  -moz-user-select: text;
  -ms-user-select: text;
  -o-user-select: text;
}
.w2ui-grid .w2ui-grid-body div.w2ui-input {
  user-select: tex;
  -webkit-user-select: tex;
  -moz-user-select: tex;
  -ms-user-select: tex;
  -o-user-select: tex;
  background-color: white;
  padding: 4px 2px;
  border: 1px solid transparent;
  width: 100%;
  height: 100%;
  pointer-events: auto;
  outline: none;
  white-space: pre;
  overflow: hidden;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-columns,
.w2ui-grid .w2ui-grid-body .w2ui-grid-fcolumns {
  overflow: hidden;
  position: absolute;
  left: 0px;
  top: 0px;
  right: 0px;
  box-shadow: 0px 1px 4px #dddddd;
  height: auto;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-columns table,
.w2ui-grid .w2ui-grid-body .w2ui-grid-fcolumns table {
  height: auto;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-columns .w2ui-resizer,
.w2ui-grid .w2ui-grid-body .w2ui-grid-fcolumns .w2ui-resizer {
  position: absolute;
  z-index: 1000;
  display: block;
  background-image: none;
  background-color: rgba(0, 0, 0, 0);
  /* needed for IE */
  padding: 0px;
  margin: 0px;
  width: 6px;
  height: 12px;
  cursor: col-resize;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords {
  position: absolute;
  left: 0px;
  right: 0px;
  top: 0px;
  bottom: 0px;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-odd,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-odd {
  color: inherit;
  background-color: #ffffff;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-odd:hover,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-odd:hover,
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-odd.w2ui-record-hover,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-odd.w2ui-record-hover {
  color: inherit;
  background-color: #e6f0ff;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-odd.w2ui-empty-record:hover,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-odd.w2ui-empty-record:hover {
  background-color: #ffffff;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-even,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-even {
  color: inherit;
  background-color: #f3f6fa;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-even:hover,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-even:hover,
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-even.w2ui-record-hover,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-even.w2ui-record-hover {
  color: inherit;
  background-color: #e6f0ff;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-even.w2ui-empty-record:hover,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-even.w2ui-empty-record:hover {
  background-color: #f3f6fa;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-selected,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-selected,
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr td.w2ui-selected,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords table tr td.w2ui-selected {
  color: #000000 !important;
  background-color: #b6d5ff !important;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr.w2ui-inactive,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-inactive,
.w2ui-grid .w2ui-grid-body .w2ui-grid-records table tr td.w2ui-inactive,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords table tr td.w2ui-inactive {
  background-color: #d8dee7 !important;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-expanded1,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords .w2ui-expanded1 {
  height: 0px;
  border-bottom: 1px solid #b2bac0;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-expanded1 > div,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords .w2ui-expanded1 > div {
  height: 0px;
  border: 0px;
  transition: height .3s, opacity .3s;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-expanded2,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords .w2ui-expanded2 {
  height: 0px;
  border-radius: 0px;
  border-bottom: 1px solid #b2bac0;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-expanded2 > div,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords .w2ui-expanded2 > div {
  height: 0px;
  border: 0px;
  transition: height .3s, opacity .3s;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-load-more,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords .w2ui-load-more {
  border-top: 1px solid #d6d5d7;
  cursor: pointer;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-load-more > div,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords .w2ui-load-more > div {
  text-align: center;
  color: #777777;
  background-color: rgba(233, 237, 243, 0.5);
  padding: 10px 0px 15px 0px;
  border-top: 1px solid #ffffff;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-records .w2ui-load-more > div:hover,
.w2ui-grid .w2ui-grid-body .w2ui-grid-frecords .w2ui-load-more > div:hover {
  color: inherit;
  background-color: #e6f0ff;
}
.w2ui-grid .w2ui-grid-body table {
  border-spacing: 0px;
  border-collapse: collapse;
  table-layout: fixed;
  width: 1px;
}
.w2ui-grid .w2ui-grid-body table .w2ui-head {
  margin: 0px;
  padding: 0px;
  border-right: 1px solid #c5c5c5;
  border-bottom: 1px solid #c5c5c5;
  color: #000000;
  background-image: -webkit-linear-gradient(#f9f9f9, #e4e4e4);
  background-image: -moz-linear-gradient(#f9f9f9, #e4e4e4);
  background-image: -ms-linear-gradient(#f9f9f9, #e4e4e4);
  background-image: -o-linear-gradient(#f9f9f9, #e4e4e4);
  background-image: linear-gradient(#f9f9f9, #e4e4e4);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#fff9f9f9', endColorstr='#ffe4e4e4', GradientType=0);
}
.w2ui-grid .w2ui-grid-body table .w2ui-head > div {
  padding: 7px 3px;
  white-space: nowrap;
  text-overflow: ellipsis;
  overflow: hidden;
  position: relative;
}
.w2ui-grid .w2ui-grid-body table .w2ui-head.w2ui-col-intersection {
  border-right-color: #72b2ff;
}
.w2ui-grid .w2ui-grid-body table .w2ui-head.w2ui-reorder-cols-head:hover {
  cursor: move;
}
.w2ui-grid .w2ui-grid-body table .w2ui-head .col-intersection-marker {
  padding: 0;
  position: absolute;
  height: 100%;
  top: 0;
}
.w2ui-grid .w2ui-grid-body table .w2ui-head .col-intersection-marker.left {
  left: 0;
  margin-left: -5px;
}
.w2ui-grid .w2ui-grid-body table .w2ui-head .col-intersection-marker.right {
  right: 0;
  margin-right: -5px;
}
.w2ui-grid .w2ui-grid-body table .w2ui-head .col-intersection-marker .top-marker {
  position: absolute;
  top: 0;
  height: 0;
  width: 0;
  border-top: 5px solid #72b2ff;
  border-left: 5px solid transparent;
  border-right: 5px solid transparent;
}
.w2ui-grid .w2ui-grid-body table .w2ui-head .col-intersection-marker .bottom-marker {
  position: absolute;
  bottom: 0;
  height: 0;
  width: 0;
  border-bottom: 5px solid #72b2ff;
  border-left: 5px solid transparent;
  border-right: 5px solid transparent;
}
.w2ui-grid .w2ui-grid-body table td {
  border-right: 1px solid #d6d5d7;
  border-bottom: 0px solid #d6d5d7;
  cursor: default;
  overflow: hidden;
}
.w2ui-grid .w2ui-grid-body table td.w2ui-soft-span,
.w2ui-grid .w2ui-grid-body table td.w2ui-soft-hidden {
  border-right-color: transparent;
}
.w2ui-grid .w2ui-grid-body table td.w2ui-grid-data {
  margin: 0px;
  padding: 0px;
}
.w2ui-grid .w2ui-grid-body table td.w2ui-grid-data .w2ui-info {
  position: relative;
  top: 1px;
  font-size: 14px;
  color: #8D99A7;
  cursor: pointer;
  width: 16px;
  display: inline-block;
  margin-right: 2px;
  text-align: center;
}
.w2ui-grid .w2ui-grid-body table td.w2ui-grid-data > div {
  padding: 3px 3px 3px 3px;
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
}
.w2ui-grid .w2ui-grid-body table td.w2ui-grid-data > div.flexible-record {
  height: auto;
  overflow: visible;
  white-space: normal;
}
.w2ui-grid .w2ui-grid-body table td.w2ui-grid-data .w2ui-show-children {
  width: 16px;
  height: 10px;
  display: inline-block;
  position: relative;
  top: -1px;
  cursor: pointer;
}
.w2ui-grid .w2ui-grid-body table td:last-child {
  border-right: 0px;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-number {
  width: 34px;
  color: #777777;
  background-color: rgba(233, 237, 243, 0.5);
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-number div {
  padding: 0px 7px 0px 3px;
  text-align: right;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-number.w2ui-head {
  cursor: pointer;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-select {
  width: 26px;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-select div {
  padding: 0px 0px;
  text-align: center;
  overflow: hidden;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-select div input[type=checkbox] {
  margin-top: 0px;
  margin-bottom: 0px;
  position: relative;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-expand {
  width: 26px;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-expand div {
  padding: 0px 0px;
  text-align: center;
  font-weight: bold;
}
.w2ui-grid .w2ui-grid-body table .w2ui-col-selected {
  background-color: #d1d1d1 !important;
}
.w2ui-grid .w2ui-grid-body table .w2ui-row-selected {
  background-color: #d1d1d1 !important;
}
.w2ui-grid .w2ui-grid-body div.w2ui-col-header {
  height: auto !important;
  width: 100%;
  overflow: hidden;
  padding-right: 10px !important;
}
.w2ui-grid .w2ui-grid-body div.w2ui-col-header > div.w2ui-sort-up {
  border: 4px solid transparent;
  border-bottom: 5px solid #8D99A7;
  margin-top: -2px;
  margin-right: -7px;
  float: right;
}
.w2ui-grid .w2ui-grid-body div.w2ui-col-header > div.w2ui-sort-down {
  border: 4px solid transparent;
  border-top: 5px solid #8D99A7;
  margin-top: 2px;
  margin-right: -7px;
  float: right;
}
.w2ui-grid .w2ui-grid-body .w2ui-col-group {
  text-align: center;
}
.w2ui-grid .w2ui-grid-body .w2ui-grid-scroll1 {
  position: absolute;
  left: 0px;
  bottom: 0px;
  border-top: 1px solid #ddd;
  border-right: 1px solid #ddd;
  background-color: #FAFAFA;
}
.w2ui-grid .w2ui-grid-empty-msg {
  position: absolute;
  top: 27px;
  left: 0;
  right: 0;
  bottom: 0;
  background-color: rgba(255, 255, 255, 0.65);
}
.w2ui-grid .w2ui-grid-empty-msg > div {
  position: absolute;
  left: 0;
  right: 0;
  top: 45%;
  transform: translateY(-45%);
  text-align: center;
  font-size: 13px;
  color: #666;
}
.w2ui-grid .w2ui-changed {
  background: url(data:image/gif;base64,R0lGODlhCgAKAJEAALAABf///wAAAAAAACH5BAEAAAIALAAAAAAKAAoAAAIPlI8Hy8mbxIsSUnup3rQAADs=) no-repeat top right;
}
.w2ui-grid .w2ui-edit-box {
  position: absolute;
  z-index: 1001;
  border: 2px solid #6299DA;
  pointer-events: auto;
}
.w2ui-grid .w2ui-editable {
  overflow: hidden;
  height: 100% !important;
  margin: 0px !important;
  padding: 0px !important;
}
.w2ui-grid .w2ui-editable input {
  border: 0px;
  border-radius: 0px;
  margin: 0px;
  padding: 4px 3px;
  width: 100%;
  height: 100%;
}
.w2ui-grid .w2ui-editable input.w2ui-select {
  outline: none !important;
  background: #fff;
}
.w2ui-grid .w2ui-grid-summary {
  position: absolute;
  box-shadow: 0px -1px 4px #aaaaaa;
}
.w2ui-grid .w2ui-grid-summary table {
  color: inherit;
}
.w2ui-grid .w2ui-grid-summary table .w2ui-odd {
  background-color: #eef5eb;
}
.w2ui-grid .w2ui-grid-summary table .w2ui-even {
  background-color: #f8fff5;
}
.w2ui-grid .w2ui-grid-footer {
  position: absolute;
  margin: 0px;
  padding: 0px;
  text-align: center;
  height: 24px;
  overflow: hidden;
  user-select: text;
  -webkit-user-select: text;
  -moz-user-select: text;
  -ms-user-select: text;
  -o-user-select: text;
  box-shadow: 0px -1px 4px #eeeeee;
  color: #444444;
  background-color: #f8f8f8;
  border-top: 1px solid #dddddd;
  border-bottom-left-radius: 2px;
  border-bottom-right-radius: 2px;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-left {
  float: left;
  padding-top: 5px;
  padding-left: 5px;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-right {
  float: right;
  padding-top: 5px;
  padding-right: 5px;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-center {
  padding: 2px;
  text-align: center;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-center .w2ui-footer-nav {
  width: 110px;
  margin: 0 auto;
  padding: 0px;
  text-align: center;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-center .w2ui-footer-nav input[type=text] {
  padding: 1px 2px 2px 2px;
  border-radius: 3px;
  width: 40px;
  text-align: center;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-center .w2ui-footer-nav a.w2ui-footer-btn {
  display: inline-block;
  border-radius: 3px;
  cursor: pointer;
  font-size: 11px;
  line-height: 16px;
  padding: 1px 5px;
  width: 30px;
  height: 18px;
  margin-top: -1px;
  color: #000000;
  background-color: transparent;
}
.w2ui-grid .w2ui-grid-footer .w2ui-footer-center .w2ui-footer-nav a.w2ui-footer-btn:hover {
  color: #000000;
  background-color: #aec8ff;
}
.w2ui-grid .w2ui-grid-focus-input {
  position: absolute;
  top: 0;
  right: 0;
  z-index: -1;
  opacity: 0;
  overflow: hidden;
  padding: 0px;
  margin: 0px;
  width: 1px;
  height: 1px;
  resize: none;
  border: 0px;
}
/* SpeadSheet */
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table tr td.w2ui-selected {
  background-color: #EEF4FE !important;
}
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table tr td.w2ui-inactive {
  background-color: #F4F6F9 !important;
}
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table td {
  border-right-width: 1px;
  border-bottom: 1px solid #efefef;
}
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table tr.w2ui-odd,
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table tr.w2ui-even,
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table tr.w2ui-odd:hover,
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table tr.w2ui-even:hover {
  background-color: inherit;
}
.w2ui-ss .w2ui-grid-body .w2ui-grid-records table tr:first-child td {
  border-top: 0px;
  border-bottom: 0px;
}
.w2ui-ss .w2ui-grid-body .w2ui-grid-frecords table tr td.w2ui-selected {
  background-color: #EEF4FE !important;
}
.w2ui-ss .w2ui-grid-body .w2ui-grid-frecords table tr td.w2ui-inactive {
  background-color: #F4F6F9 !important;
}
.w2ui-ss .w2ui-grid-body .w2ui-grid-frecords table td {
  border-right-width: 1px;
  border-bottom: 1px solid #efefef;
}
.w2ui-ss .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-odd,
.w2ui-ss .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-even,
.w2ui-ss .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-odd:hover,
.w2ui-ss .w2ui-grid-body .w2ui-grid-frecords table tr.w2ui-even:hover {
  background-color: inherit;
}
.w2ui-ss .w2ui-grid-body .w2ui-grid-frecords table tr:first-child td {
  border-bottom: 0px;
}
.w2ui-ss .w2ui-grid-body .w2ui-selection {
  position: absolute;
  z-index: 1000;
  border: 2px solid #6299DA;
  /* #457FC2; */
  pointer-events: none;
}
.w2ui-ss .w2ui-grid-body .w2ui-selection .w2ui-selection-resizer {
  cursor: crosshair;
  position: absolute;
  bottom: 0px;
  right: 0px;
  width: 6px;
  height: 6px;
  margin-right: -3px;
  margin-bottom: -3px;
  background-color: #457FC2;
  border: 0.5px solid #fff;
  outline: 1px solid white;
  pointer-events: auto;
}
.w2ui-ss .w2ui-grid-body .w2ui-selection.w2ui-inactive {
  border: 2px solid #C0C2C5;
}
.w2ui-ss .w2ui-grid-body .w2ui-selection.w2ui-inactive .w2ui-selection-resizer {
  background-color: #B0B0B0;
}
.w2ui-ss .w2ui-grid-body .w2ui-soft-range {
  position: absolute;
  pointer-events: none;
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
}
.w2ui-ss .w2ui-grid-body .w2ui-changed {
  background: inherit;
  /* do not show changed for spreadsheet */
}
.w2ui-info-bubble table {
  font-size: 11px;
  font-family: Verdana, Arial, sans-serif;
  color: white;
  text-shadow: 1px 1px solid #999;
}
.w2ui-info-bubble table tr td:first-child {
  white-space: nowrap;
  padding: 2px;
  padding-right: 10px;
  color: #ddd;
  vertical-align: top;
}
.w2ui-info-bubble table tr td:last-child {
  white-space: pre;
  padding: 2px;
}
.w2ui-overlay .w2ui-select-field {
  padding: 4px 0px;
  cursor: default;
}
.w2ui-overlay .w2ui-select-field table {
  font-size: 11px;
  font-family: Verdana, Arial, sans-serif;
  border-spacing: 0px;
  border-collapse: border-collapse;
}
.w2ui-overlay .w2ui-select-field table tr {
  height: 27px;
}
.w2ui-overlay .w2ui-select-field table tr:hover {
  background-color: #b6d5ff;
}
.w2ui-overlay .w2ui-select-field table tr td:nth-child(1) {
  width: 26px;
  padding-right: 6px;
  text-align: right;
  color: #888;
}
.w2ui-overlay .w2ui-select-field table tr td:nth-child(2) {
  padding: 3px 20px 3px 0px;
}
.w2ui-overlay .w2ui-col-on-off {
  padding: 4px 0px;
}
.w2ui-overlay .w2ui-col-on-off table {
  border-spacing: 0px;
  border-collapse: border-collapse;
}
.w2ui-overlay .w2ui-col-on-off table tr:hover {
  background-color: #b6d5ff;
}
.w2ui-overlay .w2ui-col-on-off table td input[type=checkbox] {
  margin: 3px 2px 2px 2px;
}
.w2ui-overlay .w2ui-col-on-off table td label {
  display: block;
  padding: 3px 0px;
  padding-right: 10px;
}
.w2ui-overlay .w2ui-col-on-off table td:first-child {
  padding: 4px 0px 4px 6px;
}
.w2ui-overlay .w2ui-col-on-off table td:last-child {
  padding: 4px 6px 4px 0px;
}
.w2ui-overlay .w2ui-grid-searches {
  text-align: left;
  padding: 0px;
  border-top: 0px;
  background-color: #f7f6f0;
}
.w2ui-overlay .w2ui-grid-searches table {
  padding: 4px;
  padding-top: 12px;
  border-collapse: border-collapse;
}
.w2ui-overlay .w2ui-grid-searches table td {
  padding: 4px;
  /* for IE */
}
.w2ui-overlay .w2ui-grid-searches table td.close-btn {
  width: 20px;
  padding-right: 20px;
}
.w2ui-overlay .w2ui-grid-searches table td.close-btn button {
  min-width: 24px;
  height: 24px;
  padding-top: 5px !important;
}
.w2ui-overlay .w2ui-grid-searches table td.caption {
  text-align: right;
  padding-right: 5px;
  border-right: 1px solid #e8e8e3;
}
.w2ui-overlay .w2ui-grid-searches table td.operator {
  text-align: left;
  padding: 0px 10px;
  padding-right: 5px;
  border-right: 1px solid #e8e8e3;
  height: 31px;
}
.w2ui-overlay .w2ui-grid-searches table td.operator select {
  width: 100%;
  color: black;
}
.w2ui-overlay .w2ui-grid-searches table td.operator select::-ms-expand {
  display: none;
}
.w2ui-overlay .w2ui-grid-searches table td.value {
  padding-right: 5px;
  padding-left: 5px;
}
.w2ui-overlay .w2ui-grid-searches table td.value input[type=text] {
  border-radius: 3px;
  padding: 3px;
  margin-right: 3px;
  height: 23px;
}
.w2ui-overlay .w2ui-grid-searches table td.value select {
  padding: 3px;
  margin-right: 3px;
  height: 23px;
}
.w2ui-overlay .w2ui-grid-searches table td.actions {
  border-right: 0px;
}
.w2ui-overlay .w2ui-grid-searches table td.actions > div {
  margin: -7px;
  margin-top: 15px;
  padding: 13px 0px;
  text-align: center;
  background-color: #efefe9;
  border-top: 1px solid #e8e8e3;
}
/*************************************************
* ---- Popup ----
*/
.w2ui-popup {
  position: fixed;
  z-index: 1600;
  overflow: hidden;
  font-family: Verdana, Arial, sans-serif;
  border-radius: 6px;
  padding: 0px;
  margin: 0px;
  border: 1px solid #777777;
  background-color: #eeeeee;
  box-shadow: 0px 0px 25px #555555;
}
.w2ui-popup,
.w2ui-popup * {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-popup .w2ui-popup-title {
  padding: 6px;
  border-radius: 6px 6px 0px 0px;
  background-image: -webkit-linear-gradient(#ececec, #dfdfdf);
  background-image: -moz-linear-gradient(#ececec, #dfdfdf);
  background-image: -ms-linear-gradient(#ececec, #dfdfdf);
  background-image: -o-linear-gradient(#ececec, #dfdfdf);
  background-image: linear-gradient(#ececec, #dfdfdf);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ffececec', endColorstr='#ffdfdfdf', GradientType=0);
  border-bottom: 2px solid #bfbfbf;
  position: absolute;
  overflow: hidden;
  height: 32px;
  left: 0px;
  right: 0px;
  top: 0px;
  text-overflow: ellipsis;
  text-align: center;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
  cursor: move;
  font-size: 15px;
  color: #555555;
  z-index: 300;
}
.w2ui-popup .w2ui-popup-button {
  float: right;
  width: 18px;
  height: 18px;
  cursor: pointer;
  overflow: hidden;
  padding: 0px;
  margin: 0px 3px 0px 0px;
  background: url(data:image/png;base64, ) no-repeat center left;
  background-position: 0px 0px;
  color: transparent !important;
  border-radius: 3px;
  border: 1px solid transparent;
}
.w2ui-popup .w2ui-popup-close {
  margin-top: 0px;
  background-position: -32px 0px;
}
.w2ui-popup .w2ui-popup-close:hover {
  background-color: #cccccc;
  border: 1px solid #aaaaaa;
}
.w2ui-popup .w2ui-popup-max {
  background-position: -16px 0px;
}
.w2ui-popup .w2ui-popup-max:hover {
  background-color: #cccccc;
  border: 1px solid #aaaaaa;
}
.w2ui-popup .w2ui-box,
.w2ui-popup .w2ui-box-temp {
  position: absolute;
  left: 0px;
  right: 0px;
  top: 32px;
  bottom: 52px;
  z-index: 100;
}
.w2ui-popup .w2ui-popup-body {
  font-size: 12px;
  line-height: 130%;
  padding: 0px 7px 7px 7px;
  color: #000000;
  background-color: #eeeeee;
  position: absolute;
  overflow: auto;
  width: 100%;
  height: 100%;
}
.w2ui-popup .w2ui-popup-buttons {
  padding: 12px;
  border-radius: 0px 0px 6px 6px;
  border-top: 1px solid #d5d8d8;
  background-color: #f1f1f1;
  text-align: center;
  position: absolute;
  overflow: hidden;
  height: 52px;
  left: 0px;
  right: 0px;
  bottom: 0px;
  z-index: 200;
}
.w2ui-popup .w2ui-popup-no-title {
  border-top-left-radius: 6px;
  border-top-right-radius: 6px;
  top: 0px !important;
}
.w2ui-popup .w2ui-popup-no-buttons {
  border-bottom-left-radius: 6px;
  border-bottom-right-radius: 6px;
  bottom: 0px !important;
}
.w2ui-popup .w2ui-alert-msg,
.w2ui-popup .w2ui-confirm-msg {
  font-size: 13px;
  line-height: 1.5;
}
/*************************************************
* ---- Sidebar ----
*/
.w2ui-sidebar {
  position: relative;
  cursor: default;
  overflow: hidden !important;
  background-color: #edf1f6 !important;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-sidebar * {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-sidebar > div {
  position: absolute;
  overflow: hidden;
}
.w2ui-sidebar .w2ui-sidebar-top {
  position: absolute;
  z-index: 2;
  top: 0px;
  left: 0px;
  right: 0px;
}
.w2ui-sidebar .w2ui-sidebar-top .w2ui-flat-left,
.w2ui-sidebar .w2ui-sidebar-top .w2ui-flat-right {
  position: absolute;
  right: 2px;
  top: 2px;
  height: 24px;
  padding: 5px;
  border-radius: 2px;
  background-size: 16px 12px;
  background-position: center center;
  background-repeat: no-repeat;
  background-color: #edf1f6;
}
.w2ui-sidebar .w2ui-sidebar-top .w2ui-flat-left:hover,
.w2ui-sidebar .w2ui-sidebar-top .w2ui-flat-right:hover {
  background-color: #d7e1ef;
}
.w2ui-sidebar .w2ui-sidebar-top .w2ui-flat-left {
  left: auto;
  width: 25px;
  background-image: url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAQAAADZc7J/AAAAzklEQVR4Ae2THRDEMBCFzy1ucatb3eJ2uhi3uNUtbnGrW9zi1rOdNzdvdl7nDpvYt/9/r7+/51myZZf/zXkD2iMHHRSb0x3oskwMieK05PwEXqP4ExSL0wp0ROao2OOuMPOMdUL6XU1/oGLcFWb+NqyTd2W/P/2qTr9h+nFXhOkHXRHiNyjrgp/U/V+WaQcaNY13zZI0A1JvcVqAnrGDTdtDtZUHjHIJhxxVLN0iqXgCP1l/7h8U9kc6abyJ4/eNWPpGdBv+XdUK0K8cnvcBly2rDr7C1HQAAAAASUVORK5CYII=');
}
.w2ui-sidebar .w2ui-sidebar-top .w2ui-flat-right {
  left: 2px;
  width: auto;
  background-image: url('data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACAAAAAgCAQAAADZc7J/AAAAz0lEQVR4Ae2UG7TGMBCEr1vd4la3uMUtuli3utWtbnGLW9zi9l/bDMzJG7u12cfJfLunf1+UEC9Bv0vVQwJ8hjRCaZafflb1C9RQf4OD0gSDE+i+PiJAabFhQc1y1AYYsJGLY3lgxM17uWPO56yPiFDqVPWgRtpIHSd1zPnwkBsdI58OlNwx4fP2X0TgfMTOoHSdKOXkpyNvEyQh7ul+4swxJSTQuwNDxz68l/ukVNbu0Neen5Z+KvzWxBAqHds349uPFJ/jVOrPjxUq++OLf+20q5+noXo0AAAAAElFTkSuQmCC');
}
.w2ui-sidebar .w2ui-sidebar-bottom {
  position: absolute;
  z-index: 2;
  bottom: 0px;
  left: 0px;
  right: 0px;
}
.w2ui-sidebar .w2ui-sidebar-div {
  position: absolute;
  z-index: 1;
  overflow: auto;
  top: 0px;
  bottom: 0px;
  left: 0px;
  right: 0px;
  padding: 2px 0px;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
}
.w2ui-sidebar .w2ui-sidebar-div table {
  width: 100%;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node {
  background-color: #edf1f6;
  border-top: 1px solid transparent;
  border-bottom: 1px solid transparent;
  margin: 0px;
  padding: 1px 0px;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node table {
  pointer-events: none;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-caption,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-image,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-image > span,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node td.w2ui-node-dots {
  color: #000000;
  text-shadow: 0px 0px 0px #ffffff;
  pointer-events: none;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-caption:hover,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-image:hover,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-image > span:hover,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node td.w2ui-node-dots:hover {
  color: inherit;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node:hover {
  border-top: 1px solid #f9f9f9;
  border-bottom: 1px solid #f9f9f9;
  background-color: #d7e1ef;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-image {
  width: 22px;
  text-align: center;
  pointer-events: none;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node .w2ui-node-image > span {
  color: #516173 !important;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node input,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node button {
  pointer-events: auto;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected:hover {
  position: relative;
  background-image: -webkit-linear-gradient(#69b1e0, #4a96d3);
  background-image: -moz-linear-gradient(#69b1e0, #4a96d3);
  background-image: -ms-linear-gradient(#69b1e0, #4a96d3);
  background-image: -o-linear-gradient(#69b1e0, #4a96d3);
  background-image: linear-gradient(#69b1e0, #4a96d3);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ff69b1e0', endColorstr='#ff4a96d3', GradientType=0);
  border-top: 1px solid #5295cd;
  border-bottom: 1px solid #2661a6;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected .w2ui-node-caption,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected:hover .w2ui-node-caption,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected .w2ui-node-image,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected:hover .w2ui-node-image,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected .w2ui-node-image > span,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected:hover .w2ui-node-image > span,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected td.w2ui-node-dots,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected:hover td.w2ui-node-dots {
  color: #ffffff !important;
  text-shadow: 1px 1px 2px #666666 !important;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected:before {
  content: "";
  border: 1px dashed #eee;
  border-radius: 3px;
  position: absolute;
  top: 2px;
  bottom: 2px;
  left: 2px;
  right: 2px;
  pointer-events: none;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-selected.w2ui-inactive:before {
  border: 1px dashed transparent !important;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled:hover {
  background: transparent !important;
  border-top: 1px solid transparent;
  border-bottom: 1px solid transparent;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled .w2ui-node-caption,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled:hover .w2ui-node-caption,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled .w2ui-node-image,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled:hover .w2ui-node-image,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled .w2ui-node-image > span,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled:hover .w2ui-node-image > span,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled td.w2ui-node-dots,
.w2ui-sidebar .w2ui-sidebar-div .w2ui-disabled:hover td.w2ui-node-dots {
  opacity: 0.4;
  filter: alpha(opacity=40);
  color: #000000 !important;
  text-shadow: 0px 0px 0px #ffffff !important;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-caption {
  white-space: nowrap;
  padding: 5px 0px 5px 3px;
  margin: 1px 0px 1px 22px;
  position: relative;
  z-index: 1;
  font-size: 12px;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-group {
  white-space: nowrap;
  overflow: hidden;
  padding: 10px 0px 10px 10px;
  margin: 0px;
  cursor: default;
  color: #868b92;
  background-color: transparent;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-group :nth-child(1) {
  /* show / hide link */
  margin-right: 10px;
  float: right;
  color: transparent;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-group :nth-child(2) {
  /* title text */
  font-weight: normal;
  text-transform: uppercase;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-sub {
  overflow: hidden;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-flat {
  padding: 6px 0px;
  text-align: center;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-flat .w2ui-node-image {
  text-align: center;
  width: auto;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-flat .w2ui-node-image > span {
  font-size: 16px;
  color: #000000;
  text-shadow: 0px 0px 0px #ffffff;
}
.w2ui-sidebar .w2ui-sidebar-div .w2ui-node-flat .w2ui-node-image.w2ui-icon {
  width: 21px;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-dots {
  width: 18px;
  padding: 0px 0px 1px 7px;
  text-align: center;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-dots .w2ui-expand {
  width: 16px;
  margin-top: -3px;
  pointer-events: auto;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-data {
  padding: 1px 1px 3px 1px;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-data .w2ui-node-image {
  padding: 3px 0px 0px 0px;
  float: left;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-data .w2ui-node-image > span {
  font-size: 16px;
  color: #000000;
  text-shadow: 0px 0px 0px #ffffff;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-data .w2ui-node-image.w2ui-icon {
  margin-top: 3px;
}
.w2ui-sidebar .w2ui-sidebar-div td.w2ui-node-data .w2ui-node-count {
  float: right;
  border: 1px solid #6f9dc7;
  border-radius: 20px;
  width: auto;
  height: 18px;
  padding: 2px 7px;
  margin: 3px 4px -2px 0;
  background-color: #e7f0fc;
  color: #667274;
  box-shadow: 0 0 0px #ffffff;
  text-shadow: 1px 1px 1px #e6e6e6;
  position: relative;
  z-index: 2;
}
/*************************************************
* ---- Tabs ----
*/
.w2ui-tabs {
  cursor: default;
  overflow: hidden !important;
  position: relative;
  background-color: #fafafa;
  padding: 3px 0px;
}
.w2ui-tabs:not(.w2ui-tabs-up) {
  padding-bottom: 0px !important;
}
.w2ui-tabs.w2ui-tabs-up {
  padding-top: 0px !important;
}
.w2ui-tabs table {
  border-bottom: 1px solid silver;
  padding: 0px 7px;
}
.w2ui-tabs .w2ui-tab {
  padding: 6px 20px;
  text-align: center;
  color: #000000;
  background-color: transparent;
  border: 1px solid #c0c0c0;
  border-bottom: 1px solid silver;
  white-space: nowrap;
  margin: 1px 1px -1px 0px;
  border-top-left-radius: 4px;
  border-top-right-radius: 4px;
  cursor: default;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
}
.w2ui-tabs .w2ui-tab.active {
  color: #000000;
  background-color: #ffffff;
  border: 1px solid #c0c0c0;
  border-bottom: 1px solid transparent;
}
.w2ui-tabs .w2ui-tab.closable {
  padding: 6px 28px 6px 20px;
}
.w2ui-tabs .w2ui-tab-close {
  color: #555;
  text-shadow: 1px 1px 1px #bbb;
  float: right;
  margin: 6px 4px 0px 0px;
  padding: 0px 0px 0px 5px;
  width: 16px;
  height: 16px;
  opacity: 0.9;
  border: 0px;
  border-top: 3px solid transparent;
  border-radius: 9px;
}
.w2ui-tabs .w2ui-tab-close:hover {
  background-color: #D77F7F;
  color: white;
}
.w2ui-tabs .w2ui-tab-close:before {
  position: relative;
  top: -2px;
  left: 0px;
  opacity: 0.6;
  color: inherit;
  text-shadow: inherit;
  content: 'x';
}
.w2ui-tabs.w2ui-tabs-up table {
  border-top: 1px solid #c0c0c0;
  border-bottom: 0px;
}
.w2ui-tabs.w2ui-tabs-up .w2ui-tab {
  border-top: 1px solid #C0C0C0;
  margin: -1px 1px 0px 0px;
  border-radius: 0px;
  border-bottom-left-radius: 4px;
  border-bottom-right-radius: 4px;
}
.w2ui-tabs.w2ui-tabs-up .w2ui-tab.active {
  border: 1px solid #c0c0c0;
  border-top: 1px solid transparent;
}
.w2ui-tabs.w2ui-tabs-up .w2ui-tab-close {
  margin-top: 4px;
}
/*************************************************
* ---- Toolbar ----
*/
.w2ui-toolbar {
  margin: 0px;
  padding: 2px;
  outline: 0px;
  position: relative;
  background-color: #efefef;
  overflow: hidden !important;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
}
.w2ui-toolbar .disabled {
  opacity: 0.3;
  filter: alpha(opacity=30);
}
.w2ui-toolbar table {
  table-layout: auto !important;
}
.w2ui-toolbar table td {
  border: 0px !important;
}
.w2ui-toolbar table.w2ui-button {
  margin: 0 1px 0 0;
  border-radius: 4px;
  height: 24px;
  min-width: 24px;
  border: 1px solid transparent;
  background-color: transparent;
}
.w2ui-toolbar table.w2ui-button .w2ui-tb-image {
  width: 22px;
  height: 16px;
  padding: 0;
  margin: 5px 1px 3px 1px!important;
  border: 0!important;
  text-align: center;
}
.w2ui-toolbar table.w2ui-button .w2ui-tb-image > span {
  font-size: 15px;
  display: block;
  color: #8d99a7;
}
.w2ui-toolbar table.w2ui-button .w2ui-tb-image > span:before {
  vertical-align: top !important;
}
.w2ui-toolbar table.w2ui-button .w2ui-tb-caption {
  color: #000000;
  padding: 0px 4px 0px 2px;
}
.w2ui-toolbar table.w2ui-button .w2ui-tb-count {
  padding: 0px 3px 0px 1px;
}
.w2ui-toolbar table.w2ui-button .w2ui-tb-count > span {
  border: 1px solid #f6fcf4;
  border-radius: 11px;
  width: auto;
  height: 18px;
  padding: 0px 6px 1px 6px;
  background-color: #f2f8f0;
  color: #666666;
  box-shadow: 0px 0px 2px #474545;
  text-shadow: 1px 1px 0px #ffffff;
}
.w2ui-toolbar table.w2ui-button .w2ui-tb-down {
  padding: 1px 3px 1px 1px;
}
.w2ui-toolbar table.w2ui-button .w2ui-tb-down > div {
  border: 4px solid transparent;
  border-top: 5px solid #8D99A7;
  margin-top: 5px;
}
.w2ui-toolbar table.w2ui-button.over {
  border: 1px solid #cccccc;
  background-color: #eeeeee;
}
.w2ui-toolbar table.w2ui-button.over .w2ui-tb-caption {
  color: #000000;
}
.w2ui-toolbar table.w2ui-button.down {
  border: 1px solid #aaaaaa;
  background-color: #dddddd;
}
.w2ui-toolbar table.w2ui-button.down .w2ui-tb-caption {
  color: #666666;
}
.w2ui-toolbar table.w2ui-button.checked {
  border: 1px solid #aaaaaa;
  background-color: #ffffff;
}
.w2ui-toolbar table.w2ui-button.checked .w2ui-tb-caption {
  color: #000000;
}
.w2ui-toolbar table.w2ui-button table {
  height: 17px;
  border-radius: 4px;
  cursor: default;
}
.w2ui-toolbar .w2ui-break {
  background-image: -webkit-linear-gradient(top, rgba(153, 153, 153, 0.1) 0%, #999999 40%, #999999 60%, rgba(153, 153, 153, 0.1) 100%);
  background-image: -moz-linear-gradient(top, rgba(153, 153, 153, 0.1) 0%, #999999 40%, #999999 60%, rgba(153, 153, 153, 0.1) 100%);
  background-image: -ms-linear-gradient(top, rgba(153, 153, 153, 0.1) 0%, #999999 40%, #999999 60%, rgba(153, 153, 153, 0.1) 100%);
  background-image: -o-linear-gradient(top, rgba(153, 153, 153, 0.1) 0%, #999999 40%, #999999 60%, rgba(153, 153, 153, 0.1) 100%);
  background-image: linear-gradient(top, rgba(153, 153, 153, 0.1) 0%, #999999 40%, #999999 60%, rgba(153, 153, 153, 0.1) 100%);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ff999999', endColorstr='#ff999999', GradientType=0);
  width: 1px !important;
  height: 22px;
  padding: 0px;
  margin: 0px 6px;
}
.w2ui-listview {
  overflow: auto !important;
  background-color: #ffffff !important;
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-listview * {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  -o-box-sizing: border-box;
  box-sizing: border-box;
}
.w2ui-listview > ul {
  list-style-type: none;
  margin: 0;
  cursor: default;
}
.w2ui-listview > ul > li {
  display: inline-block;
  vertical-align: top;
  overflow: hidden;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  -o-user-select: none;
  user-select: none;
  border: 1px solid transparent;
  border-radius: 4px;
}
.w2ui-listview > ul > li.w2ui-focused {
  border: 1px solid #2661a6;
}
.w2ui-listview > ul > li.w2ui-selected {
  border: 1px solid #2661a6;
}
.w2ui-listview > ul > li.w2ui-selected,
.w2ui-listview > ul > li.w2ui-selected.hover {
  background-image: -webkit-linear-gradient(#69b1e0, #4a96d3);
  background-image: -moz-linear-gradient(#69b1e0, #4a96d3);
  background-image: -ms-linear-gradient(#69b1e0, #4a96d3);
  background-image: -o-linear-gradient(#69b1e0, #4a96d3);
  background-image: linear-gradient(#69b1e0, #4a96d3);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ff69b1e0', endColorstr='#ff4a96d3', GradientType=0);
}
.w2ui-listview > ul > li.w2ui-selected > div > div.caption,
.w2ui-listview > ul > li.w2ui-selected.hover > div > div.caption {
  color: #ffffff;
}
.w2ui-listview > ul > li.w2ui-selected > div > div.description,
.w2ui-listview > ul > li.w2ui-selected.hover > div > div.description {
  color: #dddddd;
}
.w2ui-listview > ul > li.w2ui-selected > div > div.extra > div > div,
.w2ui-listview > ul > li.w2ui-selected.hover > div > div.extra > div > div {
  color: #dddddd;
}
.w2ui-listview > ul > li.hover {
  background-color: #d7e1ef;
  border: 1px solid #2661a6;
}
.w2ui-listview > ul > li div {
  vertical-align: middle;
}
.w2ui-listview > ul > li > div > div.caption {
  display: block;
  text-align: center;
  word-wrap: break-word;
  max-height: 50px;
  color: #000000;
  font-size: 12px;
}
.w2ui-listview > ul > li > div > div.description {
  display: none;
  text-align: left;
  color: #777777;
  font-size: 12px;
}
.w2ui-listview > ul > li > div > div.extra {
  display: none;
}
.w2ui-listview > ul > li > div > div.extra > div > div {
  color: #777777;
}
.w2ui-icon-small > ul {
  padding: 1px 0px 0px 1px;
}
.w2ui-icon-small > ul > li {
  margin: 0px 1px 1px 0px;
  padding: 2px;
  width: 250px;
  white-space: nowrap;
}
.w2ui-icon-small > ul > li > div > div.w2ui-listview-img {
  display: inline-block;
  width: 26px;
  height: 22px;
  font-size: 21px;
  margin-right: 2px;
}
.w2ui-icon-small > ul > li > div > div.caption {
  display: inline-block;
}
.w2ui-icon-medium > ul {
  padding: 4px 0px 0px 4px;
}
.w2ui-icon-medium > ul > li {
  margin: 0px 4px 4px 0px;
  padding: 4px;
  width: 100px;
}
.w2ui-icon-medium > ul > li > div > div.w2ui-listview-img {
  display: block;
  width: 92px;
  height: 60px;
  font-size: 57px;
  margin-left: auto;
  margin-right: auto;
  background-position: center;
}
.w2ui-icon-large > ul {
  padding: 4px 0px 0px 4px;
}
.w2ui-icon-large > ul > li {
  margin: 0px 4px 4px 0px;
  padding: 4px;
  width: 160px;
}
.w2ui-icon-large > ul > li > div > div.w2ui-listview-img {
  display: block;
  width: 152px;
  height: 120px;
  font-size: 114px;
  margin-left: auto;
  margin-right: auto;
  background-position: center;
}
.w2ui-icon-tile > ul {
  padding: 1px 0px 0px 1px;
}
.w2ui-icon-tile > ul > li {
  margin: 0px 1px 1px 0px;
  padding: 4px;
  width: 250px;
  white-space: nowrap;
}
.w2ui-icon-tile > ul > li > div > div.w2ui-listview-img {
  display: inline-block;
  width: 72px;
  height: 60px;
  font-size: 57px;
  float: left;
  margin-right: 4px;
}
.w2ui-icon-tile > ul > li > div > div.caption {
  text-align: left;
}
.w2ui-icon-tile > ul > li > div > div.description {
  display: block;
}
.w2ui-table > ul {
  padding: 0;
}
.w2ui-table > ul > li {
  width: 100%;
  padding: 2px;
  border-radius: 0px;
  border-bottom: 1px dotted lightgray;
}
.w2ui-table > ul > li > div {
  display: inline-block;
  position: relative;
  width: 100%;
  white-space: nowrap;
  overflow: hidden;
}
.w2ui-table > ul > li > div > div.w2ui-listview-img {
  display: inline-block;
  width: 38px;
  height: 32px;
  font-size: 31px;
  margin-right: 2px;
}
.w2ui-table > ul > li > div > div.caption {
  display: inline-block;
}
.w2ui-table > ul > li > div > div.extra {
  display: inline-block;
  position: absolute;
  right: 0;
  height: 100%;
  background-color: #ffffff;
}
.w2ui-table > ul > li > div > div.extra > div:before {
  display: inline-block;
  height: 100%;
  width: 0;
  content: '';
  vertical-align: middle;
}
.w2ui-table > ul > li > div > div.extra > div {
  display: inline;
}
.w2ui-table > ul > li > div > div.extra > div > div {
  display: inline-block;
  font-size: 12px;
}
.w2ui-table > ul > li.w2ui-selected div.extra,
.w2ui-table > ul > li.w2ui-selected.hover div.extra {
  background-image: -webkit-linear-gradient(#69b1e0, #4a96d3);
  background-image: -moz-linear-gradient(#69b1e0, #4a96d3);
  background-image: -ms-linear-gradient(#69b1e0, #4a96d3);
  background-image: -o-linear-gradient(#69b1e0, #4a96d3);
  background-image: linear-gradient(#69b1e0, #4a96d3);
  filter: progid:dximagetransform.microsoft.gradient(startColorstr='#ff69b1e0', endColorstr='#ff4a96d3', GradientType=0);
}
.w2ui-table > ul > li.hover div.extra {
  background-color: #d7e1ef;
}
.w2ui-listview > ul > li div.icon-none {
  border: 1px solid rgba(102, 102, 102, 0.35);
}
