<!DOCTYPE HTML>
<html lang="en" xmlns="http://www.w3.org/1999/html">
<head>
    <meta charset="UTF-8">
    <title>{{title or ''}}</title>

    <link rel="stylesheet" href="http://{{my_ip}}/static/w2ui/w2ui-1.4.3.min.css">
    <link rel="stylesheet" href="http://{{my_ip}}/static/bootstrap-3.3.7/css/bootstrap.min.css">
    <link rel="stylesheet" href="http://{{my_ip}}/static/bootstrap-3.3.7/css/bootstrap-theme.min.css">
    <link rel="stylesheet" href="http://{{my_ip}}/static//strf_web.css">
    <script src="http://{{my_ip}}/static/jquery/jquery-3.2.1.min.js"></script>
    <script src="http://{{my_ip}}/static/bootstrap-3.3.7/js/bootstrap.min.js"></script>
    <script src="http://{{my_ip}}/static/jsmol/JSmol_dk.nojq.lite.js"></script>
    <script src="http://{{my_ip}}/static/w2ui/w2ui-1.4.3.min.js"></script>
    <script src="http://{{my_ip}}/static/clipboard/clipboard.min.js"></script>
    <script src="http://{{my_ip}}/static/strf_web.js"></script>
    <script>
        var cgifile = 'http://{{my_ip}}';
    </script>


</head>

<body>

{{!base}}



</body>


</html>